@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -22,12 +22,6 @@
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef WIN32
-#include "config-win32.h"
-#else
-#include "config.h"
-#endif
-
 #include "syshead.h"
 
 #include "error.h"
@@ -41,6 +35,7 @@
 #include "perf.h"
 #include "status.h"
 #include "integer.h"
+#include "ps.h"
 
 #ifdef USE_CRYPTO
 #include <openssl/err.h>
@@ -83,11 +78,26 @@ static bool use_syslog;     /* GLOBAL */
 static bool suppress_timestamps; /* GLOBAL */
 
 /* The program name passed to syslog */
+#if SYSLOG_CAPABILITY
 static char *pgmname_syslog;  /* GLOBAL */
+#endif
 
 /* If non-null, messages should be written here (used for debugging only) */
 static FILE *msgfp;         /* GLOBAL */
 
+/* If true, we forked from main OpenVPN process */
+static bool forked;         /* GLOBAL */
+
+/* our default output targets */
+static FILE *default_out; /* GLOBAL */
+static FILE *default_err; /* GLOBAL */
+
+void
+msg_forked (void)
+{
+  forked = true;
+}
+
 bool
 set_debug_level (const int level, const unsigned int flags)
 {
@@ -145,6 +155,8 @@ error_reset ()
   mute_cutoff = 0;
   mute_count = 0;
   mute_category = 0;
+  default_out = OPENVPN_MSG_FP;
+  default_err = OPENVPN_MSG_FP;
 
 #ifdef OPENVPN_DEBUG_COMMAND_LINE
   msgfp = fopen (OPENVPN_DEBUG_FILE, "w");
@@ -155,15 +167,21 @@ error_reset ()
 #endif
 }
 
+void
+errors_to_stderr (void)
+{
+  default_err = OPENVPN_ERROR_FP;  
+}
+
 /*
  * Return a file to print messages to before syslog is opened.
  */
 FILE *
-msg_fp()
+msg_fp(const unsigned int flags)
 {
   FILE *fp = msgfp;
   if (!fp)
-    fp = OPENVPN_MSG_FP;
+    fp = (flags & (M_FATAL|M_USAGE_SMALL)) ? default_err : default_out;
   if (!fp)
     openvpn_exit (OPENVPN_EXIT_STATUS_CANNOT_OPEN_DEBUG_FILE); /* exit point */
   return fp;
@@ -264,27 +282,31 @@ void x_msg (const unsigned int flags, const char *format, ...)
 #endif
 
   /* set up client prefix */
-  prefix = msg_get_prefix ();
+  if (flags & M_NOIPREFIX)
+    prefix = NULL;
+  else
+    prefix = msg_get_prefix ();
   prefix_sep = " ";
   if (!prefix)
     prefix_sep = prefix = "";
 
   /* virtual output capability used to copy output to management subsystem */
-  {
-    const struct virtual_output *vo = msg_get_virtual_output ();
-    if (vo)
-      {
-	openvpn_snprintf (m2, ERR_BUF_SIZE, "%s%s%s",
-			  prefix,
-			  prefix_sep,
-			  m1);
-	virtual_output_print (vo, flags, m2);
-      }
-  }
+  if (!forked)
+    {
+      const struct virtual_output *vo = msg_get_virtual_output ();
+      if (vo)
+	{
+	  openvpn_snprintf (m2, ERR_BUF_SIZE, "%s%s%s",
+			    prefix,
+			    prefix_sep,
+			    m1);
+	  virtual_output_print (vo, flags, m2);
+	}
+    }
 
   if (!(flags & M_MSG_VIRT_OUT))
     {
-      if (use_syslog && !std_redir)
+      if (use_syslog && !std_redir && !forked)
 	{
 #if SYSLOG_CAPABILITY
 	  syslog (level, "%s%s%s",
@@ -295,31 +317,34 @@ void x_msg (const unsigned int flags, const char *format, ...)
 	}
       else
 	{
-	  FILE *fp = msg_fp();
+	  FILE *fp = msg_fp(flags);
 	  const bool show_usec = check_debug_level (DEBUG_LEVEL_USEC_TIME);
 
 	  if ((flags & M_NOPREFIX) || suppress_timestamps)
 	    {
-	      fprintf (fp, "%s%s%s\n",
+	      fprintf (fp, "%s%s%s%s",
 		       prefix,
 		       prefix_sep,
-		       m1);
+		       m1,
+		       (flags&M_NOLF) ? "" : "\n");
 	    }
 	  else
 	    {
 #ifdef USE_PTHREAD
-	      fprintf (fp, "%s [%d] %s%s%s\n",
+	      fprintf (fp, "%s [%d] %s%s%s%s",
 		       time_string (0, 0, show_usec, &gc),
 		       (int) openvpn_thread_self (),
 		       prefix,
 		       prefix_sep,
-		       m1);
+		       m1,
+		       (flags&M_NOLF) ? "" : "\n");
 #else
-	      fprintf (fp, "%s %s%s%s\n",
+	      fprintf (fp, "%s %s%s%s%s",
 		       time_string (0, 0, show_usec, &gc),
 		       prefix,
 		       prefix_sep,
-		       m1);
+		       m1,
+		       (flags&M_NOLF) ? "" : "\n");
 #endif
 	    }
 	  fflush(fp);
@@ -564,7 +589,7 @@ x_check_status (int status,
 		struct link_socket *sock,
 		struct tuntap *tt)
 {
-  const int my_errno = (sock ? openvpn_errno_socket () : openvpn_errno ());
+  const int my_errno = (sock ? openvpn_errno_socket () : (int)openvpn_errno ());
   const char *extended_msg = NULL;
 
   msg (x_cs_verbose_level, "%s %s returned %d",
@@ -671,6 +696,11 @@ openvpn_exit (const int status)
   plugin_abort ();
 #endif
 
+#if PORT_SHARE
+  if (port_share)
+    port_share_abort (port_share);
+#endif
+
 #ifdef ABORT_ON_ERROR
   if (status == OPENVPN_EXIT_STATUS_ERROR)
     abort ();
@@ -702,6 +732,15 @@ msg_flags_string (const unsigned int flags, struct gc_arena *gc)
   return BSTR (&out);
 }
 
+#ifdef ENABLE_DEBUG
+void
+crash (void)
+{
+  char *null = NULL;
+  *null = 0;
+}
+#endif
+
 #ifdef WIN32
 
 const char *
