@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -22,12 +22,6 @@
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef WIN32
-#include "config-win32.h"
-#else
-#include "config.h"
-#endif
-
 #include "syshead.h"
 
 #include "fdmisc.h"
@@ -36,25 +30,43 @@
 #include "memdbg.h"
 
 /* Set a file descriptor to non-blocking */
-void
-set_nonblock (int fd)
+bool
+set_nonblock_action (int fd)
 {
 #ifdef WIN32
   u_long arg = 1;
   if (ioctlsocket (fd, FIONBIO, &arg))
-    msg (M_SOCKERR, "Set socket to non-blocking mode failed");
+    return false;
 #else
   if (fcntl (fd, F_SETFL, O_NONBLOCK) < 0)
-    msg (M_ERR, "Set file descriptor to non-blocking mode failed");
+    return false;
 #endif
+  return true;
+}
+
+/* Set a file descriptor to not be passed across execs */
+bool
+set_cloexec_action (int fd)
+{
+#ifndef WIN32
+  if (fcntl (fd, F_SETFD, FD_CLOEXEC) < 0)
+    return false;
+#endif
+  return true;
+}
+
+/* Set a file descriptor to non-blocking */
+void
+set_nonblock (int fd)
+{
+  if (!set_nonblock_action (fd))
+    msg (M_SOCKERR, "Set socket to non-blocking mode failed");
 }
 
 /* Set a file descriptor to not be passed across execs */
 void
 set_cloexec (int fd)
 {
-#ifndef WIN32
-  if (fcntl (fd, F_SETFD, FD_CLOEXEC) < 0)
+  if (!set_cloexec_action (fd))
     msg (M_ERR, "Set FD_CLOEXEC flag on file descriptor failed");
-#endif
 }
