@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -22,12 +22,6 @@
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef WIN32
-#include "config-win32.h"
-#else
-#include "config.h"
-#endif
-
 #include "syshead.h"
 
 #include "forward.h"
@@ -36,6 +30,9 @@
 #include "gremlin.h"
 #include "mss.h"
 #include "event.h"
+#include "ps.h"
+#include "dhcp.h"
+#include "common.h"
 
 #include "memdbg.h"
 
@@ -43,6 +40,9 @@
 #include "occ-inline.h"
 #include "ping-inline.h"
 
+counter_type link_read_bytes_global;  /* GLOBAL */
+counter_type link_write_bytes_global; /* GLOBAL */
+
 /* show event wait debugging info */
 
 #ifdef ENABLE_DEBUG
@@ -88,13 +88,19 @@ check_tls_dowork (struct context *c)
 
   if (interval_test (&c->c2.tmp_int))
     {
-      if (tls_multi_process
-	  (c->c2.tls_multi, &c->c2.to_link, &c->c2.to_link_addr,
-	   get_link_socket_info (c), &wakeup))
+      const int tmp_status = tls_multi_process
+	(c->c2.tls_multi, &c->c2.to_link, &c->c2.to_link_addr,
+	 get_link_socket_info (c), &wakeup);
+      if (tmp_status == TLSMP_ACTIVE)
 	{
 	  update_time ();
 	  interval_action (&c->c2.tmp_int);
 	}
+      else if (tmp_status == TLSMP_KILL)
+	{
+	  c->sig->signal_received = SIGTERM;
+	  c->sig->signal_text = "auth-control-exit";
+	}
 
       interval_future_trigger (&c->c2.tmp_int, wakeup);
     }
@@ -151,6 +157,8 @@ check_incoming_control_channel_dowork (struct context *c)
 	    receive_auth_failed (c, &buf);
 	  else if (buf_string_match_head_str (&buf, "PUSH_"))
 	    incoming_push_message (c, &buf);
+	  else if (buf_string_match_head_str (&buf, "RESTART"))
+	    server_pushed_restart (c, &buf);
 	  else
 	    msg (D_PUSH_ERRORS, "WARNING: Received unknown control message: %s", BSTR (&buf));
 	}
@@ -170,9 +178,12 @@ void
 check_push_request_dowork (struct context *c)
 {
   send_push_request (c);
+
+  /* if no response to first push_request, retry at 5 second intervals */
+  event_timeout_modify_wakeup (&c->c2.push_request_interval, 5);
 }
 
-#endif
+#endif /* P2MP */
 
 /*
  * Things that need to happen immediately after connection initiation should go here.
@@ -194,13 +205,12 @@ check_connection_established_dowork (struct context *c)
 		  management_set_state (management,
 					OPENVPN_STATE_GET_CONFIG,
 					NULL,
+					0,
 					0);
 		}
 #endif
-	      send_push_request (c);
-
-	      /* if no reply, try again in 5 sec */
-	      event_timeout_init (&c->c2.push_request_interval, 5, now);
+	      /* send push request in 1 sec */
+	      event_timeout_init (&c->c2.push_request_interval, 1, now);
 	      reset_coarse_timers (c);
 	    }
 	  else
@@ -252,7 +262,7 @@ send_control_channel_string (struct context *c, const char *str, int msglevel)
 static void
 check_add_routes_action (struct context *c, const bool errors)
 {
-  do_route (&c->options, c->c1.route_list, c->c1.tuntap, c->c1.plugins, c->c2.es);
+  do_route (&c->options, c->c1.route_list, c->c1.tuntap, c->plugins, c->c2.es);
   update_time ();
   event_timeout_clear (&c->c2.route_wakeup);
   event_timeout_clear (&c->c2.route_wakeup_expire);
@@ -273,8 +283,23 @@ check_add_routes_dowork (struct context *c)
   else
     {
       msg (D_ROUTE, "Route: Waiting for TUN/TAP interface to come up...");
+      if (c->c1.tuntap)
+	{
+	  if (!tun_standby (c->c1.tuntap))
+	    {
+	      c->sig->signal_received = SIGHUP;
+	      c->sig->signal_text = "ip-fail";
+	      c->persist.restart_sleep_seconds = 10;
+#ifdef WIN32
+	      show_routes (M_INFO|M_NOPREFIX);
+	      show_adapters (M_INFO|M_NOPREFIX);
+#endif
+	    }
+	}
+      update_time ();
       if (c->c2.route_wakeup.n != 1)
 	event_timeout_init (&c->c2.route_wakeup, 1, now);
+      event_timeout_reset (&c->c2.ping_rec_interval);
     }
 }
 
@@ -291,16 +316,30 @@ check_inactivity_timeout_dowork (struct context *c)
 
 #if P2MP
 
+void
+check_server_poll_timeout_dowork (struct context *c)
+{
+  event_timeout_reset (&c->c2.server_poll_interval);
+  if (!tls_initial_packet_received (c->c2.tls_multi))
+    {
+      msg (M_INFO, "Server poll timeout, restarting");
+      c->sig->signal_received = SIGUSR1;
+      c->sig->signal_text = "server_poll";
+      c->persist.restart_sleep_seconds = -1;
+    }
+}
+
 /*
- * Schedule a SIGTERM n_seconds from now.
+ * Schedule a signal n_seconds from now.
  */
 void
-schedule_exit (struct context *c, const int n_seconds)
+schedule_exit (struct context *c, const int n_seconds, const int signal)
 {
   tls_set_single_session (c->c2.tls_multi);
   update_time ();
   reset_coarse_timers (c);
   event_timeout_init (&c->c2.scheduled_exit, n_seconds, now);
+  c->c2.scheduled_exit_signal = signal;
   msg (D_SCHED_EXIT, "Delayed exit in %d seconds", n_seconds);
 }
 
@@ -310,7 +349,7 @@ schedule_exit (struct context *c, const int n_seconds)
 void
 check_scheduled_exit_dowork (struct context *c)
 {
-  c->sig->signal_received = SIGTERM;
+  c->sig->signal_received = c->c2.scheduled_exit_signal;
   c->sig->signal_text = "delayed-exit";
 }
 
@@ -339,7 +378,7 @@ check_fragment_dowork (struct context *c)
   if (lsi->mtu_changed && c->c2.ipv4_tun)
     {
       frame_adjust_path_mtu (&c->c2.frame_fragment, c->c2.link_socket->mtu,
-			     c->options.proto);
+			     c->options.ce.proto);
       lsi->mtu_changed = false;
     }
 
@@ -398,7 +437,7 @@ encrypt_sign (struct context *c, bool comp_frag)
     {
 #ifdef USE_LZO
       /* Compress the packet. */
-      if (c->options.comp_lzo)
+      if (lzo_defined (&c->c2.lzo_compwork))
 	lzo_compress (&c->c2.buf, b->lzo_compress_buf, &c->c2.lzo_compwork, &c->c2.frame);
 #endif
 #ifdef ENABLE_FRAGMENT
@@ -475,6 +514,10 @@ process_coarse_timers (struct context *c)
   check_push_request (c);
 #endif
 
+#ifdef PLUGIN_PF
+  pf_check_reload (c);
+#endif
+
   /* process --route options */
   check_add_routes (c);
 
@@ -489,6 +532,10 @@ process_coarse_timers (struct context *c)
     return;
 
 #if P2MP
+  check_server_poll_timeout (c);
+  if (c->sig->signal_received)
+    return;
+
   check_scheduled_exit (c);
   if (c->sig->signal_received)
     return;
@@ -572,12 +619,12 @@ socks_postprocess_incoming_link (struct context *c)
 
 static inline void
 socks_preprocess_outgoing_link (struct context *c,
-				struct sockaddr_in **to_addr,
+				struct link_socket_actual **to_addr,
 				int *size_delta)
 {
   if (c->c2.link_socket->socks_proxy && c->c2.link_socket->info.proto == PROTO_UDPv4)
     {
-      *size_delta += socks_process_outgoing_udp (&c->c2.to_link, &c->c2.to_link_addr);
+      *size_delta += socks_process_outgoing_udp (&c->c2.to_link, c->c2.to_link_addr);
       *to_addr = &c->c2.link_socket->socks_relay;
     }
 }
@@ -616,22 +663,39 @@ read_incoming_link (struct context *c)
 
   c->c2.buf = c->c2.buffers->read_link_buf;
   ASSERT (buf_init (&c->c2.buf, FRAME_HEADROOM_ADJ (&c->c2.frame, FRAME_HEADROOM_MARKER_READ_LINK)));
-  status = link_socket_read (c->c2.link_socket, &c->c2.buf, MAX_RW_SIZE_LINK (&c->c2.frame), &c->c2.from);
+
+  status = link_socket_read (c->c2.link_socket,
+			     &c->c2.buf,
+			     MAX_RW_SIZE_LINK (&c->c2.frame),
+			     &c->c2.from);
 
   if (socket_connection_reset (c->c2.link_socket, status))
     {
-      /* received a disconnect from a connection-oriented protocol */
-      if (c->options.inetd)
+#if PORT_SHARE
+      if (port_share && socket_foreign_protocol_detected (c->c2.link_socket))
 	{
+	  const struct buffer *fbuf = socket_foreign_protocol_head (c->c2.link_socket);
+	  const int sd = socket_foreign_protocol_sd (c->c2.link_socket);
+	  port_share_redirect (port_share, fbuf, sd);
 	  c->sig->signal_received = SIGTERM;
-	  msg (D_STREAM_ERRORS, "Connection reset, inetd/xinetd exit [%d]", status);
+	  c->sig->signal_text = "port-share-redirect";
 	}
       else
-	{
-	  c->sig->signal_received = SIGUSR1; /* SOFT-SIGUSR1 -- TCP connection reset */
-	  msg (D_STREAM_ERRORS, "Connection reset, restarting [%d]", status);
-	}
-      c->sig->signal_text = "connection-reset";
+#endif
+      {
+	/* received a disconnect from a connection-oriented protocol */
+	if (c->options.inetd)
+	  {
+	    c->sig->signal_received = SIGTERM;
+	    msg (D_STREAM_ERRORS, "Connection reset, inetd/xinetd exit [%d]", status);
+	  }
+	else
+	  {
+	    c->sig->signal_received = SIGUSR1; /* SOFT-SIGUSR1 -- TCP connection reset */
+	    msg (D_STREAM_ERRORS, "Connection reset, restarting [%d]", status);
+	  }
+	c->sig->signal_text = "connection-reset";
+      }
       perf_pop ();
       return;
     }
@@ -665,11 +729,21 @@ process_incoming_link (struct context *c)
   if (c->c2.buf.len > 0)
     {
       c->c2.link_read_bytes += c->c2.buf.len;
+      link_read_bytes_global += c->c2.buf.len;
       c->c2.original_recv_size = c->c2.buf.len;
+#ifdef ENABLE_MANAGEMENT
+      if (management)
+	{
+	  management_bytes_in (management, c->c2.buf.len);
+#ifdef MANAGEMENT_DEF_AUTH
+	  management_bytes_server (management, &c->c2.link_read_bytes, &c->c2.link_write_bytes, &c->c2.mda_context);
+#endif
+	}
+#endif
     }
   else
     c->c2.original_recv_size = 0;
-
+  
 #ifdef ENABLE_DEBUG
   /* take action to corrupt packet if we are in gremlin test mode */
   if (c->options.gremlin) {
@@ -687,7 +761,7 @@ process_incoming_link (struct context *c)
   msg (D_LINK_RW, "%s READ [%d] from %s: %s",
        proto2ascii (lsi->proto, true),
        BLEN (&c->c2.buf),
-       print_sockaddr (&c->c2.from, &gc),
+       print_link_socket_actual (&c->c2.from, &gc),
        PROTO_DUMP (&c->c2.buf, &gc));
 
   /*
@@ -764,9 +838,19 @@ process_incoming_link (struct context *c)
 
 #ifdef USE_LZO
       /* decompress the incoming packet */
-      if (c->options.comp_lzo)
+      if (lzo_defined (&c->c2.lzo_compwork))
 	lzo_decompress (&c->c2.buf, c->c2.buffers->lzo_decompress_buf, &c->c2.lzo_compwork, &c->c2.frame);
 #endif
+
+#ifdef PACKET_TRUNCATION_CHECK
+      /* if (c->c2.buf.len > 1) --c->c2.buf.len; */
+      ipv4_packet_size_verify (BPTR (&c->c2.buf),
+			       BLEN (&c->c2.buf),
+			       TUNNEL_TYPE (c->c1.tuntap),
+			       "POST_DECRYPT",
+			       &c->c2.n_trunc_post_decrypt);
+#endif
+
       /*
        * Set our "official" outgoing address, since
        * if buf.len is non-zero, we know the packet
@@ -793,7 +877,7 @@ process_incoming_link (struct context *c)
       /* Did we just receive an openvpn ping packet? */
       if (is_ping_msg (&c->c2.buf))
 	{
-	  dmsg (D_PACKET_CONTENT, "RECEIVED PING PACKET");
+	  dmsg (D_PING, "RECEIVED PING PACKET");
 	  c->c2.buf.len = 0; /* drop packet */
 	}
 
@@ -841,6 +925,14 @@ read_incoming_tun (struct context *c)
   c->c2.buf.len = read_tun (c->c1.tuntap, BPTR (&c->c2.buf), MAX_RW_SIZE_TUN (&c->c2.frame));
 #endif
 
+#ifdef PACKET_TRUNCATION_CHECK
+  ipv4_packet_size_verify (BPTR (&c->c2.buf),
+			   BLEN (&c->c2.buf),
+			   TUNNEL_TYPE (c->c1.tuntap),
+			   "READ_TUN",
+			   &c->c2.n_trunc_tun_read);
+#endif
+
   /* Was TUN/TAP interface stopped? */
   if (tuntap_stop (c->c2.buf.len))
     {
@@ -887,6 +979,16 @@ process_incoming_tun (struct context *c)
        * us to examine the IPv4 header.
        */
       process_ipv4_header (c, PIPV4_PASSTOS|PIPV4_MSSFIX, &c->c2.buf);
+
+#ifdef PACKET_TRUNCATION_CHECK
+      /* if (c->c2.buf.len > 1) --c->c2.buf.len; */
+      ipv4_packet_size_verify (BPTR (&c->c2.buf),
+			       BLEN (&c->c2.buf),
+			       TUNNEL_TYPE (c->c1.tuntap),
+			       "PRE_ENCRYPT",
+			       &c->c2.n_trunc_pre_encrypt);
+#endif
+
       encrypt_sign (c, true);
     }
   else
@@ -906,6 +1008,8 @@ process_ipv4_header (struct context *c, unsigned int flags, struct buffer *buf)
   if (!c->options.passtos)
     flags &= ~PIPV4_PASSTOS;
 #endif
+  if (!c->options.route_gateway_via_dhcp || !route_list_default_gateway_needed (c->c1.route_list))
+    flags &= ~PIPV4_EXTRACT_DHCP_ROUTER;
 
   if (buf->len > 0)
     {
@@ -931,6 +1035,14 @@ process_ipv4_header (struct context *c, unsigned int flags, struct buffer *buf)
 	      /* possibly alter the TCP MSS */
 	      if (flags & PIPV4_MSSFIX)
 		mss_fixup (&ipbuf, MTU_TO_MSS (TUN_MTU_SIZE_DYNAMIC (&c->c2.frame)));
+
+	      /* possibly extract a DHCP router message */
+	      if (flags & PIPV4_EXTRACT_DHCP_ROUTER)
+		{
+		  const in_addr_t dhcp_router = dhcp_extract_router_msg (&ipbuf);
+		  if (dhcp_router)
+		    route_list_add_default_gateway (c->c1.route_list, c->c2.es, dhcp_router);
+		}
 	    }
 	}
     }
@@ -954,7 +1066,7 @@ process_outgoing_link (struct context *c)
        * packet to remote over the TCP/UDP port.
        */
       int size = 0;
-      ASSERT (addr_defined (&c->c2.to_link_addr));
+      ASSERT (link_socket_actual_defined (c->c2.to_link_addr));
 
 #ifdef ENABLE_DEBUG
       /* In gremlin-test mode, we may choose to drop this packet */
@@ -968,7 +1080,7 @@ process_outgoing_link (struct context *c)
 #ifdef HAVE_GETTIMEOFDAY
 	  if (c->options.shaper)
 	    shaper_wrote_bytes (&c->c2.shaper, BLEN (&c->c2.to_link)
-				+ datagram_overhead (c->options.proto));
+				+ datagram_overhead (c->options.ce.proto));
 #endif
 	  /*
 	   * Let the pinger know that we sent a packet.
@@ -989,12 +1101,12 @@ process_outgoing_link (struct context *c)
 	  msg (D_LINK_RW, "%s WRITE [%d] to %s: %s",
 	       proto2ascii (c->c2.link_socket->info.proto, true),
 	       BLEN (&c->c2.to_link),
-	       print_sockaddr (&c->c2.to_link_addr, &gc),
+	       print_link_socket_actual (c->c2.to_link_addr, &gc),
 	       PROTO_DUMP (&c->c2.to_link, &gc));
 
 	  /* Packet send complexified by possible Socks5 usage */
 	  {
-	    struct sockaddr_in *to_addr = &c->c2.to_link_addr;
+	    struct link_socket_actual *to_addr = c->c2.to_link_addr;
 #ifdef ENABLE_SOCKS
 	    int size_delta = 0;
 #endif
@@ -1004,7 +1116,9 @@ process_outgoing_link (struct context *c)
 	    socks_preprocess_outgoing_link (c, &to_addr, &size_delta);
 #endif
 	    /* Send packet */
-	    size = link_socket_write (c->c2.link_socket, &c->c2.to_link, to_addr);
+	    size = link_socket_write (c->c2.link_socket,
+				      &c->c2.to_link,
+				      to_addr);
 
 #ifdef ENABLE_SOCKS
 	    /* Undo effect of prepend */
@@ -1016,6 +1130,16 @@ process_outgoing_link (struct context *c)
 	    {
 	      c->c2.max_send_size_local = max_int (size, c->c2.max_send_size_local);
 	      c->c2.link_write_bytes += size;
+	      link_write_bytes_global += size;
+#ifdef ENABLE_MANAGEMENT
+	      if (management)
+		{
+		  management_bytes_out (management, size);
+#ifdef MANAGEMENT_DEF_AUTH
+		  management_bytes_server (management, &c->c2.link_read_bytes, &c->c2.link_write_bytes, &c->c2.mda_context);
+#endif
+		}
+#endif
 	    }
 	}
 
@@ -1028,16 +1152,19 @@ process_outgoing_link (struct context *c)
 	  if (size != BLEN (&c->c2.to_link))
 	    msg (D_LINK_ERRORS,
 		 "TCP/UDP packet was truncated/expanded on write to %s (tried=%d,actual=%d)",
-		 print_sockaddr (&c->c2.to_link_addr, &gc),
+		 print_link_socket_actual (c->c2.to_link_addr, &gc),
 		 BLEN (&c->c2.to_link),
 		 size);
 	}
+
+      /* indicate activity regarding --inactive parameter */
+      register_activity (c, size);
     }
   else
     {
       if (c->c2.to_link.len > 0)
 	msg (D_LINK_ERRORS, "TCP/UDP packet too large on write to %s (tried=%d,max=%d)",
-	     print_sockaddr (&c->c2.to_link_addr, &gc),
+	     print_link_socket_actual (c->c2.to_link_addr, &gc),
 	     c->c2.to_link.len,
 	     EXPANDED_SIZE (&c->c2.frame));
     }
@@ -1070,7 +1197,7 @@ process_outgoing_tun (struct context *c)
    * The --mssfix option requires
    * us to examine the IPv4 header.
    */
-  process_ipv4_header (c, PIPV4_MSSFIX, &c->c2.to_tun);
+  process_ipv4_header (c, PIPV4_MSSFIX|PIPV4_EXTRACT_DHCP_ROUTER|PIPV4_OUTGOING, &c->c2.to_tun);
 
   if (c->c2.to_tun.len <= MAX_RW_SIZE_TUN (&c->c2.frame))
     {
@@ -1085,6 +1212,14 @@ process_outgoing_tun (struct context *c)
 #endif
       dmsg (D_TUN_RW, "TUN WRITE [%d]", BLEN (&c->c2.to_tun));
 
+#ifdef PACKET_TRUNCATION_CHECK
+      ipv4_packet_size_verify (BPTR (&c->c2.to_tun),
+			       BLEN (&c->c2.to_tun),
+			       TUNNEL_TYPE (c->c1.tuntap),
+			       "WRITE_TUN",
+			       &c->c2.n_trunc_tun_write);
+#endif
+
 #ifdef TUN_PASS_BUFFER
       size = write_tun_buffered (c->c1.tuntap, &c->c2.to_tun);
 #else
@@ -1105,6 +1240,9 @@ process_outgoing_tun (struct context *c)
 		 c->c1.tuntap->actual_name,
 		 BLEN (&c->c2.to_tun),
 		 size);
+
+	  /* indicate activity regarding --inactive parameter */
+	  register_activity (c, size);
 	}
     }
   else
@@ -1118,13 +1256,6 @@ process_outgoing_tun (struct context *c)
 	   MAX_RW_SIZE_TUN (&c->c2.frame));
     }
 
-  /*
-   * Putting the --inactive timeout reset here, ensures that we will timeout
-   * if the remote goes away, even if we are trying to send data to the
-   * remote and failing.
-   */
-  register_activity (c);
-
   buf_reset (&c->c2.to_tun);
 
   perf_pop ();
@@ -1197,11 +1328,11 @@ io_wait_dowork (struct context *c, const unsigned int flags)
   struct event_set_return esr[4];
 
   /* These shifts all depend on EVENT_READ and EVENT_WRITE */
-  static const int socket_shift = 0;     /* depends on SOCKET_READ and SOCKET_WRITE */
-  static const int tun_shift = 2;        /* depends on TUN_READ and TUN_WRITE */
-  static const int err_shift = 4;        /* depends on ES_ERROR */
+  static int socket_shift = 0;     /* depends on SOCKET_READ and SOCKET_WRITE */
+  static int tun_shift = 2;        /* depends on TUN_READ and TUN_WRITE */
+  static int err_shift = 4;        /* depends on ES_ERROR */
 #ifdef ENABLE_MANAGEMENT
-  static const int management_shift = 6; /* depends on MANAGEMENT_READ and MANAGEMENT_WRITE */
+  static int management_shift = 6; /* depends on MANAGEMENT_READ and MANAGEMENT_WRITE */
 #endif
 
   /*
