@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -22,12 +22,6 @@
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef WIN32
-#include "config-win32.h"
-#else
-#include "config.h"
-#endif
-
 #include "syshead.h"
 
 #include "forward.h"
@@ -59,6 +53,14 @@ print_opt_route_gateway (const in_addr_t route_gateway, struct gc_arena *gc)
   return BSTR (&out);
 }
 
+static const char *
+print_opt_route_gateway_dhcp (struct gc_arena *gc)
+{
+  struct buffer out = alloc_buf_gc (32, gc);
+  buf_printf (&out, "route-gateway dhcp");
+  return BSTR (&out);
+}
+
 static const char *
 print_opt_route (const in_addr_t network, const in_addr_t netmask, struct gc_arena *gc)
 {
@@ -76,6 +78,16 @@ print_opt_route (const in_addr_t network, const in_addr_t netmask, struct gc_are
   return BSTR (&out);
 }
 
+static const char *
+print_opt_topology (const int topology, struct gc_arena *gc)
+{
+  struct buffer out = alloc_buf_gc (128, gc);
+
+  buf_printf (&out, "topology %s", print_topology (topology));
+
+  return BSTR (&out);
+}
+
 static const char *
 print_str_int (const char *str, const int i, struct gc_arena *gc)
 {
@@ -84,6 +96,14 @@ print_str_int (const char *str, const int i, struct gc_arena *gc)
   return BSTR (&out);
 }
 
+static const char *
+print_str (const char *str, struct gc_arena *gc)
+{
+  struct buffer out = alloc_buf_gc (128, gc);
+  buf_printf (&out, "%s", str);
+  return BSTR (&out);
+}
+
 static void
 helper_add_route (const in_addr_t network, const in_addr_t netmask, struct options *o)
 {
@@ -132,19 +152,22 @@ helper_client_server (struct options *o)
    *
    * mode server
    * tls-server
+   * push "topology [topology]"
    *
-   * if tun:
-   *   ifconfig 10.8.0.1 10.8.0.2 
-   *   ifconfig-pool 10.8.0.4 10.8.0.251
+   * if tun AND (topology == net30 OR topology == p2p):
+   *   ifconfig 10.8.0.1 10.8.0.2
+   *   if !nopool: 
+   *     ifconfig-pool 10.8.0.4 10.8.0.251
    *   route 10.8.0.0 255.255.255.0
    *   if client-to-client:
    *     push "route 10.8.0.0 255.255.255.0"
-   *   else if !linear-addr:
+   *   else if topology == net30:
    *     push "route 10.8.0.1"
    *
-   * if tap:
+   * if tap OR (tun AND topology == subnet):
    *   ifconfig 10.8.0.1 255.255.255.0
-   *   ifconfig-pool 10.8.0.2 10.8.0.254 255.255.255.0
+   *   if !nopool: 
+   *     ifconfig-pool 10.8.0.2 10.8.0.254 255.255.255.0
    *   push "route-gateway 10.8.0.1"
    */
 
@@ -152,6 +175,7 @@ helper_client_server (struct options *o)
    * Get tun/tap/null device type
    */
   const int dev = dev_type_enum (o->dev, o->dev_type);
+  const int topology = o->topology;
 
   if (o->server_defined)
     {
@@ -161,13 +185,13 @@ helper_client_server (struct options *o)
       if (o->client)
 	msg (M_USAGE, "--server and --client cannot be used together");
 
-      if (o->server_bridge_defined)
+      if (o->server_bridge_defined || o->server_bridge_proxy_dhcp)
 	msg (M_USAGE, "--server and --server-bridge cannot be used together");
 
       if (o->shared_secret_file)
 	msg (M_USAGE, "--server and --secret cannot be used together (you must use SSL/TLS keys)");
 
-      if (o->ifconfig_pool_defined)
+      if (!(o->server_flags & SF_NOPOOL) && o->ifconfig_pool_defined)
 	msg (M_USAGE, "--server already defines an ifconfig-pool, so you can't also specify --ifconfig-pool explicitly");
 
       if (!(dev == DEV_TYPE_TAP || dev == DEV_TYPE_TUN))
@@ -197,17 +221,46 @@ helper_client_server (struct options *o)
 
 	  o->mode = MODE_SERVER;
 	  o->tls_server = true;
-	  o->ifconfig_local = print_in_addr_t (o->server_network + 1, 0, &o->gc);
-	  o->ifconfig_remote_netmask = print_in_addr_t (o->server_network + 2, 0, &o->gc);
-	  o->ifconfig_pool_defined = true;
-	  o->ifconfig_pool_start = o->server_network + 4;
-	  o->ifconfig_pool_end = (o->server_network | ~o->server_netmask) - pool_end_reserve;
-	  ifconfig_pool_verify_range (M_USAGE, o->ifconfig_pool_start, o->ifconfig_pool_end);
-	  helper_add_route (o->server_network, o->server_netmask, o);
-	  if (o->enable_c2c)
-	    push_option (o, print_opt_route (o->server_network, o->server_netmask, &o->gc), M_USAGE);
-	  else if (!o->ifconfig_pool_linear)
-	    push_option (o, print_opt_route (o->server_network + 1, 0, &o->gc), M_USAGE);
+
+	  if (topology == TOP_NET30 || topology == TOP_P2P)
+	    {
+	      o->ifconfig_local = print_in_addr_t (o->server_network + 1, 0, &o->gc);
+	      o->ifconfig_remote_netmask = print_in_addr_t (o->server_network + 2, 0, &o->gc);
+
+	      if (!(o->server_flags & SF_NOPOOL))
+		{
+		  o->ifconfig_pool_defined = true;
+		  o->ifconfig_pool_start = o->server_network + 4;
+		  o->ifconfig_pool_end = (o->server_network | ~o->server_netmask) - pool_end_reserve;
+		  ifconfig_pool_verify_range (M_USAGE, o->ifconfig_pool_start, o->ifconfig_pool_end);
+		}
+
+	      helper_add_route (o->server_network, o->server_netmask, o);
+	      if (o->enable_c2c)
+		push_option (o, print_opt_route (o->server_network, o->server_netmask, &o->gc), M_USAGE);
+	      else if (topology == TOP_NET30)
+		push_option (o, print_opt_route (o->server_network + 1, 0, &o->gc), M_USAGE);
+	    }
+	  else if (topology == TOP_SUBNET)
+	    {
+	      o->ifconfig_local = print_in_addr_t (o->server_network + 1, 0, &o->gc);
+	      o->ifconfig_remote_netmask = print_in_addr_t (o->server_netmask, 0, &o->gc);
+
+	      if (!(o->server_flags & SF_NOPOOL))
+		{
+		  o->ifconfig_pool_defined = true;
+		  o->ifconfig_pool_start = o->server_network + 2;
+		  o->ifconfig_pool_end = (o->server_network | ~o->server_netmask) - 2;
+		  ifconfig_pool_verify_range (M_USAGE, o->ifconfig_pool_start, o->ifconfig_pool_end);
+		}
+	      o->ifconfig_pool_netmask = o->server_netmask;
+		  
+	      push_option (o, print_opt_route_gateway (o->server_network + 1, &o->gc), M_USAGE);
+	    }
+	  else
+	    ASSERT (0);
+
+	  push_option (o, print_opt_topology (topology, &o->gc), M_USAGE);
 	}
       else if (dev == DEV_TYPE_TAP)
 	{
@@ -219,11 +272,16 @@ helper_client_server (struct options *o)
 	  o->tls_server = true;
 	  o->ifconfig_local = print_in_addr_t (o->server_network + 1, 0, &o->gc);
 	  o->ifconfig_remote_netmask = print_in_addr_t (o->server_netmask, 0, &o->gc);
-	  o->ifconfig_pool_defined = true;
-	  o->ifconfig_pool_start = o->server_network + 2;
-	  o->ifconfig_pool_end = (o->server_network | ~o->server_netmask) - 1;
-	  ifconfig_pool_verify_range (M_USAGE, o->ifconfig_pool_start, o->ifconfig_pool_end);	  
+
+	  if (!(o->server_flags & SF_NOPOOL))
+	    {
+	      o->ifconfig_pool_defined = true;
+	      o->ifconfig_pool_start = o->server_network + 2;
+	      o->ifconfig_pool_end = (o->server_network | ~o->server_netmask) - 1;
+	      ifconfig_pool_verify_range (M_USAGE, o->ifconfig_pool_start, o->ifconfig_pool_end);
+	    }
 	  o->ifconfig_pool_netmask = o->server_netmask;
+
 	  push_option (o, print_opt_route_gateway (o->server_network + 1, &o->gc), M_USAGE);
 	}
       else
@@ -231,8 +289,13 @@ helper_client_server (struct options *o)
 	  ASSERT (0);
 	}
 
-      if (o->proto == PROTO_TCPv4)
-	o->proto = PROTO_TCPv4_SERVER;
+      /* set push-ifconfig-constraint directive */
+      if ((dev == DEV_TYPE_TAP || topology == TOP_SUBNET))
+	{
+	  o->push_ifconfig_constraint_defined = true;
+	  o->push_ifconfig_constraint_network = o->server_network;
+	  o->push_ifconfig_constraint_netmask = o->server_netmask;
+	}
     }
 
   /*
@@ -247,13 +310,25 @@ helper_client_server (struct options *o)
    *
    * ifconfig-pool 10.8.0.128 10.8.0.254 255.255.255.0
    * push "route-gateway 10.8.0.4"
+   *
+   * OR
+   *
+   * server-bridge
+   *
+   * EXPANDS TO:
+   *
+   * mode server
+   * tls-server
+   *
+   * if !nogw:
+   *   push "route-gateway dhcp"
    */
-  else if (o->server_bridge_defined)
+  else if (o->server_bridge_defined | o->server_bridge_proxy_dhcp)
     {
       if (o->client)
 	msg (M_USAGE, "--server-bridge and --client cannot be used together");
 
-      if (o->ifconfig_pool_defined)
+      if (!(o->server_flags & SF_NOPOOL) && o->ifconfig_pool_defined)
 	msg (M_USAGE, "--server-bridge already defines an ifconfig-pool, so you can't also specify --ifconfig-pool explicitly");
 
       if (o->shared_secret_file)
@@ -262,21 +337,29 @@ helper_client_server (struct options *o)
       if (dev != DEV_TYPE_TAP)
 	msg (M_USAGE, "--server-bridge directive only makes sense with --dev tap");
 
-      verify_common_subnet ("--server-bridge", o->server_bridge_ip, o->server_bridge_pool_start, o->server_bridge_netmask); 
-      verify_common_subnet ("--server-bridge", o->server_bridge_pool_start, o->server_bridge_pool_end, o->server_bridge_netmask); 
-      verify_common_subnet ("--server-bridge", o->server_bridge_ip, o->server_bridge_pool_end, o->server_bridge_netmask); 
+      if (o->server_bridge_defined)
+	{
+	  verify_common_subnet ("--server-bridge", o->server_bridge_ip, o->server_bridge_pool_start, o->server_bridge_netmask); 
+	  verify_common_subnet ("--server-bridge", o->server_bridge_pool_start, o->server_bridge_pool_end, o->server_bridge_netmask); 
+	  verify_common_subnet ("--server-bridge", o->server_bridge_ip, o->server_bridge_pool_end, o->server_bridge_netmask); 
+	}
 
       o->mode = MODE_SERVER;
       o->tls_server = true;
-      o->ifconfig_pool_defined = true;
-      o->ifconfig_pool_start = o->server_bridge_pool_start;
-      o->ifconfig_pool_end = o->server_bridge_pool_end;
-      ifconfig_pool_verify_range (M_USAGE, o->ifconfig_pool_start, o->ifconfig_pool_end);
-      o->ifconfig_pool_netmask = o->server_bridge_netmask;
-      push_option (o, print_opt_route_gateway (o->server_bridge_ip, &o->gc), M_USAGE);
 
-      if (o->proto == PROTO_TCPv4)
-	o->proto = PROTO_TCPv4_SERVER;
+      if (o->server_bridge_defined)
+	{
+	  o->ifconfig_pool_defined = true;
+	  o->ifconfig_pool_start = o->server_bridge_pool_start;
+	  o->ifconfig_pool_end = o->server_bridge_pool_end;
+	  ifconfig_pool_verify_range (M_USAGE, o->ifconfig_pool_start, o->ifconfig_pool_end);
+	  o->ifconfig_pool_netmask = o->server_bridge_netmask;
+	  push_option (o, print_opt_route_gateway (o->server_bridge_ip, &o->gc), M_USAGE);
+	}
+      else if (o->server_bridge_proxy_dhcp && !(o->server_flags & SF_NO_PUSH_ROUTE_GATEWAY))
+	{
+	  push_option (o, print_opt_route_gateway_dhcp (&o->gc), M_USAGE);
+	}
     }
   else
 #endif /* P2MP_SERVER */
@@ -298,16 +381,10 @@ helper_client_server (struct options *o)
 
       o->pull = true;
       o->tls_client = true;
-
-      if (o->proto == PROTO_TCPv4)
-	o->proto = PROTO_TCPv4_CLIENT;
     }
 
 #endif /* P2MP */
 
-  if (o->proto == PROTO_TCPv4)
-    msg (M_USAGE, "--proto tcp is ambiguous in this context.  Please specify --proto tcp-server or --proto tcp-client");
-
   gc_free (&gc);
 }
 
@@ -370,3 +447,34 @@ helper_keepalive (struct options *o)
 	}
     }
 }
+
+/*
+ *
+ * HELPER DIRECTIVE:
+ *
+ * tcp-nodelay
+ *
+ * EXPANDS TO:
+ *
+ * if mode server:
+ *   socket-flags TCP_NODELAY
+ *   push "socket-flags TCP_NODELAY"
+ */
+void
+helper_tcp_nodelay (struct options *o)
+{
+#if P2MP_SERVER
+  if (o->server_flags & SF_TCP_NODELAY_HELPER)
+    {
+      if (o->mode == MODE_SERVER)
+	{
+	  o->sockflags |= SF_TCP_NODELAY;	  
+	  push_option (o, print_str ("socket-flags TCP_NODELAY", &o->gc), M_USAGE);
+	}
+      else
+	{
+	  ASSERT (0);
+	}
+    }
+#endif
+}
