@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -22,17 +22,12 @@
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef WIN32
-#include "config-win32.h"
-#else
-#include "config.h"
-#endif
-
 #include "syshead.h"
 
 #include "init.h"
 #include "forward.h"
 #include "multi.h"
+#include "win32.h"
 
 #include "memdbg.h"
 
@@ -122,6 +117,9 @@ main (int argc, char *argv[])
        */
       do
 	{
+	  /* enter pre-initialization mode with regard to signal handling */
+	  pre_init_signal_catch ();
+
 	  /* zero context struct but leave first_time member alone */
 	  context_clear_all_except_first_time (&c);
 
@@ -133,7 +131,10 @@ main (int argc, char *argv[])
 	  gc_init (&c.gc);
 
 	  /* initialize environmental variable store */
-	  c.es = env_set_create (&c.gc);
+	  c.es = env_set_create (NULL);
+#ifdef WIN32
+	  env_set_add_win32 (c.es);
+#endif
 
 #ifdef ENABLE_MANAGEMENT
 	  /* initialize management subsystem */
@@ -141,11 +142,18 @@ main (int argc, char *argv[])
 #endif
 
 	  /* initialize options to default state */
-	  init_options (&c.options);
+	  init_options (&c.options, true);
 
 	  /* parse command line options, and read configuration file */
 	  parse_argv (&c.options, argc, argv, M_USAGE, OPT_P_DEFAULT, NULL, c.es);
 
+#ifdef ENABLE_PLUGIN
+	  /* plugins may contribute options configuration */
+	  init_verb_mute (&c, IVM_LEVEL_1);
+	  init_plugins (&c);
+	  open_plugins (&c, true, OPENVPN_PLUGIN_INIT_PRE_CONFIG_PARSE);
+#endif
+
 	  /* init verbosity and mute levels */
 	  init_verb_mute (&c, IVM_LEVEL_1);
 
@@ -165,7 +173,7 @@ main (int argc, char *argv[])
 	    break;
 
 	  /* sanity check on options */
-	  options_postprocess (&c.options, c.first_time);
+	  options_postprocess (&c.options);
 
 	  /* show all option settings */
 	  show_settings (&c.options);
@@ -185,7 +193,7 @@ main (int argc, char *argv[])
 	  if (!open_management (&c))
 	    break;
 #endif
-	  
+
 	  /* set certain options as environmental variables */
 	  setenv_settings (c.es, &c.options);
 
@@ -229,6 +237,8 @@ main (int argc, char *argv[])
 
   context_gc_free (&c);
 
+  env_set_destroy (c.es);
+
 #ifdef ENABLE_MANAGEMENT
   /* close management interface */
   close_management ();
