@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -46,6 +46,7 @@
 #include "pool.h"
 #include "plugin.h"
 #include "manage.h"
+#include "pf.h"
 
 /*
  * Our global key schedules, packaged thusly
@@ -117,6 +118,14 @@ struct context_buffers
   struct buffer read_tun_buf;
 };
 
+/*
+ * always-persistent context variables
+ */
+struct context_persist
+{
+  int restart_sleep_seconds;
+};
+
 /* 
  * level 0 context contains data related to
  * once-per OpenVPN instantiation events
@@ -149,9 +158,6 @@ struct context_1
   /* persist crypto sequence number to/from file */
   struct packet_id_persist pid_persist;
 
-  /* array of remote addresses */
-  struct remote_list *remote_list;
-
   /* TUN/TAP interface */
   struct tuntap *tuntap;
   bool tuntap_owned;
@@ -166,17 +172,15 @@ struct context_1
 #ifdef ENABLE_HTTP_PROXY
   /* HTTP proxy object */
   struct http_proxy_info *http_proxy;
+  bool http_proxy_owned;
 #endif
 
 #ifdef ENABLE_SOCKS
   /* SOCKS proxy object */
   struct socks_proxy_info *socks_proxy;
+  bool socks_proxy_owned;
 #endif
 
-  /* shared object plugins */
-  struct plugin_list *plugins;
-  bool plugins_owned;
-  
 #if P2MP
 
 #if P2MP_SERVER
@@ -185,8 +189,8 @@ struct context_1
   bool ifconfig_pool_persist_owned;
 #endif
 
-  /* if client mode, option strings we pulled from server */
-  char *pulled_options_string_save;
+  /* if client mode, hash of option strings we pulled from server */
+  struct md5_digest pulled_options_digest_save;
 
   /* save user/pass for authentication */
   struct user_pass *auth_user_pass;
@@ -226,8 +230,8 @@ struct context_2
   struct link_socket_info *link_socket_info;
   const struct link_socket *accept_from; /* possibly do accept() on a parent link_socket */
 
-  struct sockaddr_in to_link_addr;	 /* IP address of remote */
-  struct sockaddr_in from;               /* address of incoming datagram */
+  struct link_socket_actual *to_link_addr;	/* IP address of remote */
+  struct link_socket_actual from;               /* address of incoming datagram */
 
   /* MTU frame parameters */
   struct frame frame;
@@ -254,16 +258,25 @@ struct context_2
   counter_type link_read_bytes;
   counter_type link_read_bytes_auth;
   counter_type link_write_bytes;
+#ifdef PACKET_TRUNCATION_CHECK
+  counter_type n_trunc_tun_read;
+  counter_type n_trunc_tun_write;
+  counter_type n_trunc_pre_encrypt;
+  counter_type n_trunc_post_decrypt;
+#endif
 
   /*
    * Timer objects for ping and inactivity
    * timeout features.
    */
   struct event_timeout wait_for_connect;
-  struct event_timeout inactivity_interval;
   struct event_timeout ping_send_interval;
   struct event_timeout ping_rec_interval;
 
+  /* --inactive */
+  struct event_timeout inactivity_interval;
+  int inactivity_bytes;
+
 #ifdef ENABLE_OCC
   /* the option strings must match across peers */
   char *options_string_local;
@@ -390,6 +403,7 @@ struct context_2
 
   /* environmental variables to pass to scripts */
   struct env_set *es;
+  bool es_owned;
 
   /* don't wait for TUN/TAP/UDP to be ready to accept write */
   bool fast_io;
@@ -413,10 +427,25 @@ struct context_2
 #endif
 
   struct event_timeout push_request_interval;
-  const char *pulled_options_string;
+  bool did_pre_pull_restore;
+
+  /* hash of pulled options, so we can compare when options change */
+  struct md5_state pulled_options_state;
+  struct md5_digest pulled_options_digest;
+
+  struct event_timeout server_poll_interval;
 
   struct event_timeout scheduled_exit;
+  int scheduled_exit_signal;
+#endif
 
+  /* packet filter */
+#ifdef ENABLE_PF
+  struct pf_context pf;
+#endif
+
+#ifdef MANAGEMENT_DEF_AUTH
+  struct man_def_auth_context mda_context;
 #endif
 };
 
@@ -452,9 +481,16 @@ struct context
   /* signal info */
   struct signal_info *sig;
 
+  /* shared object plugins */
+  struct plugin_list *plugins;
+  bool plugins_owned;
+  
   /* set to true after we daemonize */
   bool did_we_daemonize;
 
+  /* persistent across SIGHUP */
+  struct context_persist persist;
+
   /* level 0 context contains data related to
      once-per OpenVPN instantiation events
      such as daemonization */
