@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -39,6 +39,8 @@
 #include "plugin.h"
 #include "manage.h"
 #include "proxy.h"
+#include "lzo.h"
+#include "pushlist.h"
 
 /*
  * Maximum number of parameters associated with an option,
@@ -56,17 +58,6 @@ extern const char title_string[];
 
 #if P2MP
 
-#if P2MP_SERVER
-/* parameters to be pushed to peer */
-
-#define MAX_PUSH_LIST_LEN TLS_CHANNEL_BUF_SIZE /* This parm is related to PLAINTEXT_BUFFER_SIZE in ssl.h */
-
-struct push_list {
-  /* newline delimited options, like config file */
-  char options[MAX_PUSH_LIST_LEN];
-};
-#endif
-
 /* certain options are saved before --pull modifications are applied */
 struct options_pre_pull
 {
@@ -74,17 +65,72 @@ struct options_pre_pull
   struct tuntap_options tuntap_options;
 
   bool routes_defined;
-  struct route_option_list routes;
+  struct route_option_list *routes;
 
   int foreign_option_index;
 };
 
 #endif
 
+struct connection_entry
+{
+  int proto;
+  int local_port;
+  bool local_port_defined;
+  int remote_port;
+  bool port_option_used;
+  const char *local;
+  const char *remote;
+  bool remote_float;
+  bool bind_defined;
+  bool bind_local;
+  int connect_retry_seconds;
+  bool connect_retry_defined;
+  int connect_retry_max;
+  int connect_timeout;
+  bool connect_timeout_defined;
+#ifdef ENABLE_HTTP_PROXY
+  struct http_proxy_options *http_proxy_options;
+#endif  
+#ifdef ENABLE_SOCKS
+  const char *socks_proxy_server;
+  int socks_proxy_port;
+  bool socks_proxy_retry;
+#endif
+};
+
+struct remote_entry
+{
+  const char *remote;
+  int remote_port;
+  int proto;
+};
+
+#ifdef ENABLE_CONNECTION
+
+#define CONNECTION_LIST_SIZE 64
+
+struct connection_list
+{
+  int len;
+  int current;
+  bool no_advance;
+  struct connection_entry *array[CONNECTION_LIST_SIZE];
+};
+
+struct remote_list
+{
+  int len;
+  struct remote_entry *array[CONNECTION_LIST_SIZE];
+};
+
+#endif
+
 /* Command line options */
 struct options
 {
   struct gc_arena gc;
+  bool gc_owned;
 
   /* first config file */
   const char *config;
@@ -94,6 +140,9 @@ struct options
 # define MODE_SERVER         1
   int mode;
 
+  /* enable forward compatibility for post-2.1 features */
+  bool forward_compatible;
+
   /* persist parms */
   bool persist_config;
   int persist_mode;
@@ -110,19 +159,24 @@ struct options
 #endif
 
   /* Networking parms */
-  const char *local;
-  int local_port;
-  bool local_port_defined;
-  int remote_port;
-  bool port_option_used;
-  bool remote_float;
+  struct connection_entry ce;
+
+#ifdef ENABLE_CONNECTION
+  struct connection_list *connection_list;
   struct remote_list *remote_list;
+#endif
+
+#ifdef GENERAL_PROXY_SUPPORT
+  struct auto_proxy_info *auto_proxy_info;
+#endif
+
   bool remote_random;
   const char *ipchange;
-  bool bind_local;
   const char *dev;
   const char *dev_type;
   const char *dev_node;
+  const char *lladdr;
+  int topology; /* one of the TOP_x values from proto.h */
   const char *ifconfig_local;
   const char *ifconfig_remote_netmask;
   bool ifconfig_noexec;
@@ -137,11 +191,6 @@ struct options
   bool tun_mtu_defined;  /* true if user overriding parm with command line option */
   bool link_mtu_defined; /* true if user overriding parm with command line option */
 
-  /* Protocol type (PROTO_UDP or PROTO_TCP) */
-  int proto;
-  int connect_retry_seconds;
-  bool connect_retry_defined;
-
   /* Advanced MTU negotiation and datagram fragmentation options */
   int mtu_discover_type; /* used if OS supports setting Path MTU discovery options on socket */
 
@@ -156,7 +205,9 @@ struct options
   int keepalive_ping;           /* a proxy for ping/ping-restart */
   int keepalive_timeout;
 
-  int inactivity_timeout;
+  int inactivity_timeout;       /* --inactive */
+  int inactivity_minimum_bytes;
+
   int ping_send_timeout;        /* Send a TCP/UDP ping to remote every n seconds */
   int ping_rec_timeout;         /* Expect a TCP/UDP ping from remote at least once every n seconds */
   bool ping_timer_remote;       /* Run ping timer only if we have a remote address */
@@ -192,6 +243,9 @@ struct options
   const char *groupname;
   const char *chroot_dir;
   const char *cd_dir;
+#ifdef HAVE_SETCON
+  char *selinux_context;
+#endif
   const char *writepid;
   const char *up_script;
   const char *down_script;
@@ -223,33 +277,30 @@ struct options
   bool fast_io;
 
 #ifdef USE_LZO
-  bool comp_lzo;
-  bool comp_lzo_adaptive;
+  /* LZO_x flags from lzo.h */
+  unsigned int lzo;
 #endif
 
   /* buffer sizes */
   int rcvbuf;
   int sndbuf;
 
+  /* socket flags */
+  unsigned int sockflags;
+
   /* route management */
   const char *route_script;
   const char *route_default_gateway;
+  int route_default_metric;
   bool route_noexec;
   int route_delay;
   int route_delay_window;
   bool route_delay_defined;
+  int max_routes;
   struct route_option_list *routes;
-
-#ifdef ENABLE_HTTP_PROXY
-  struct http_proxy_options *http_proxy_options;
-#endif
-
-#ifdef ENABLE_SOCKS
-  /* socks proxy */
-  const char *socks_proxy_server;
-  int socks_proxy_port;
-  bool socks_proxy_retry;
-#endif
+  bool route_nopull;
+  bool route_gateway_via_dhcp;
+  bool allow_pull_fqdn; /* as a client, allow server to push a FQDN for certain parameters */
 
 #ifdef ENABLE_OCC
   /* Enable options consistency check between peers */
@@ -263,8 +314,13 @@ struct options
   int management_log_history_cache;
   int management_echo_buffer_size;
   int management_state_buffer_size;
-  bool management_query_passwords;
-  bool management_hold;
+  const char *management_write_peer_info_file;
+
+  const char *management_client_user;
+  const char *management_client_group;
+
+  /* Mask of MF_ values of manage.h */
+  unsigned int management_flags;
 #endif
 
 #ifdef ENABLE_PLUGIN
@@ -283,20 +339,26 @@ struct options
   in_addr_t server_network;
   in_addr_t server_netmask;
 
+# define SF_NOPOOL (1<<0)
+# define SF_TCP_NODELAY_HELPER (1<<1)
+# define SF_NO_PUSH_ROUTE_GATEWAY (1<<2)
+  unsigned int server_flags;
+
+  bool server_bridge_proxy_dhcp;
+
   bool server_bridge_defined;
   in_addr_t server_bridge_ip;
   in_addr_t server_bridge_netmask;
   in_addr_t server_bridge_pool_start;
   in_addr_t server_bridge_pool_end;
 
-  struct push_list *push_list;
+  struct push_list push_list;
   bool ifconfig_pool_defined;
   in_addr_t ifconfig_pool_start;
   in_addr_t ifconfig_pool_end;
   in_addr_t ifconfig_pool_netmask;
   const char *ifconfig_pool_persist_filename;
   int ifconfig_pool_persist_refresh_freq;
-  bool ifconfig_pool_linear;
   int real_hash_size;
   int virtual_hash_size;
   const char *client_connect_script;
@@ -312,6 +374,9 @@ struct options
   bool push_ifconfig_defined;
   in_addr_t push_ifconfig_local;
   in_addr_t push_ifconfig_remote_netmask;
+  bool push_ifconfig_constraint_defined;
+  in_addr_t push_ifconfig_constraint_network;
+  in_addr_t push_ifconfig_constraint_netmask;
   bool enable_c2c;
   bool duplicate_cn;
   int cf_max;
@@ -319,17 +384,23 @@ struct options
   int max_clients;
   int max_routes_per_client;
 
-  bool client_cert_not_required;
-  bool username_as_common_name;
   const char *auth_user_pass_verify_script;
   bool auth_user_pass_verify_script_via_file;
+  unsigned int ssl_flags; /* set to SSLF_x flags from ssl.h */
+#if PORT_SHARE
+  char *port_share_host;
+  int port_share_port;
+#endif
 #endif
 
   bool client;
   bool pull; /* client pull of config options from server */
+  int push_continuation;
   const char *auth_user_pass_file;
   struct options_pre_pull *pre_pull;
 
+  int server_poll_timeout;
+
   int scheduled_exit_interval;
 
 #endif
@@ -337,12 +408,17 @@ struct options
 #ifdef USE_CRYPTO
   /* Cipher parms */
   const char *shared_secret_file;
+#if ENABLE_INLINE_FILES
+  const char *shared_secret_file_inline;
+#endif
   int key_direction;
   bool ciphername_defined;
   const char *ciphername;
   bool authname_defined;
   const char *authname;
   int keysize;
+  const char *prng_hash;
+  int prng_nonce_secret_len;
   const char *engine;
   bool replay;
   bool mute_replay_warnings;
@@ -357,6 +433,7 @@ struct options
   bool tls_server;
   bool tls_client;
   const char *ca_file;
+  const char *ca_path;
   const char *dh_file;
   const char *cert_file;
   const char *priv_key_file;
@@ -365,7 +442,28 @@ struct options
   const char *tls_verify;
   const char *tls_remote;
   const char *crl_file;
+
+#if ENABLE_INLINE_FILES
+  const char *ca_file_inline;
+  const char *cert_file_inline;
+  char *priv_key_file_inline;
+  const char *dh_file_inline;
+#endif
+
   int ns_cert_type; /* set to 0, NS_SSL_SERVER, or NS_SSL_CLIENT */
+  unsigned remote_cert_ku[MAX_PARMS];
+  const char *remote_cert_eku;
+
+#ifdef ENABLE_PKCS11
+  const char *pkcs11_providers[MAX_PARMS];
+  unsigned pkcs11_private_mode[MAX_PARMS];
+  bool pkcs11_protected_authentication[MAX_PARMS];
+  bool pkcs11_cert_private[MAX_PARMS];
+  int pkcs11_pin_cache_period;
+  const char *pkcs11_id;
+  bool pkcs11_id_management;
+#endif
+
 #ifdef WIN32
   const char *cryptoapi_cert;
 #endif
@@ -390,6 +488,9 @@ struct options
 
   /* Special authentication MAC for TLS control channel */
   const char *tls_auth_file;		/* shared secret */
+#if ENABLE_INLINE_FILES
+  const char *tls_auth_file_inline;
+#endif
 
   /* Allow only one session */
   bool single_session;
@@ -437,8 +538,14 @@ struct options
 #define OPT_P_EXPLICIT_NOTIFY (1<<19)
 #define OPT_P_ECHO            (1<<20)
 #define OPT_P_INHERIT         (1<<21)
+#define OPT_P_ROUTE_EXTRAS    (1<<22)
+#define OPT_P_PULL_MODE       (1<<23)
+#define OPT_P_PLUGIN          (1<<24)
+#define OPT_P_SOCKBUF         (1<<25)
+#define OPT_P_SOCKFLAGS       (1<<26)
+#define OPT_P_CONNECTION      (1<<27)
 
-#define OPT_P_DEFAULT   (~OPT_P_INSTANCE)
+#define OPT_P_DEFAULT   (~(OPT_P_INSTANCE|OPT_P_PULL_MODE))
 
 #if P2MP
 #define PULL_DEFINED(opt) ((opt)->pull)
@@ -473,6 +580,12 @@ struct options
 #define PLUGIN_OPTION_LIST(opt) (NULL)
 #endif
 
+#ifdef MANAGEMENT_DEF_AUTH
+#define MAN_CLIENT_AUTH_ENABLED(opt) ((opt)->management_flags & MF_CLIENT_AUTH)
+#else
+#define MAN_CLIENT_AUTH_ENABLED(opt) (false)
+#endif
+
 void parse_argv (struct options *options,
 		 const int argc,
 		 char *argv[],
@@ -485,7 +598,7 @@ void notnull (const char *arg, const char *description);
 
 void usage_small (void);
 
-void init_options (struct options *o);
+void init_options (struct options *o, const bool init_gc);
 void uninit_options (struct options *o);
 
 void setenv_settings (struct env_set *es, const struct options *o);
@@ -503,14 +616,14 @@ char *options_string (const struct options *o,
 		      bool remote,
 		      struct gc_arena *gc);
 
-int options_cmp_equal_safe (char *actual, const char *expected, size_t actual_n);
+bool options_cmp_equal_safe (char *actual, const char *expected, size_t actual_n);
 void options_warning_safe (char *actual, const char *expected, size_t actual_n);
-int options_cmp_equal (char *actual, const char *expected);
+bool options_cmp_equal (char *actual, const char *expected);
 void options_warning (char *actual, const char *expected);
 
 #endif
 
-void options_postprocess (struct options *options, bool first_time);
+void options_postprocess (struct options *options);
 
 void pre_pull_save (struct options *o);
 void pre_pull_restore (struct options *o);
@@ -545,6 +658,13 @@ int parse_line (const char *line,
 		int msglevel,
 		struct gc_arena *gc);
 
+/*
+ * parse/print topology coding
+ */
+
+int parse_topology (const char *str, const int msglevel);
+const char *print_topology (const int topology);
+
 /*
  * Manage auth-retry variable
  */
@@ -561,4 +681,33 @@ const char *auth_retry_print (void);
 
 #endif
 
+void options_string_import (struct options *options,
+			    const char *config,
+			    const int msglevel,
+			    const unsigned int permission_mask,
+			    unsigned int *option_types_found,
+			    struct env_set *es);
+
+/*
+ * inline functions
+ */
+static inline bool
+connection_list_defined (const struct options *o)
+{
+#ifdef ENABLE_CONNECTION
+  return o->connection_list != NULL;
+#else
+  return false;
+#endif
+}
+
+static inline void
+connection_list_set_no_advance (struct options *o)
+{
+#ifdef ENABLE_CONNECTION
+  if (o->connection_list)
+    o->connection_list->no_advance = true;
+#endif
+}
+
 #endif
