@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -54,14 +54,80 @@ const char* time_string (time_t t, int usec, bool show_usec, struct gc_arena *gc
 const char *tv_string (const struct timeval *tv, struct gc_arena *gc);
 const char *tv_string_abs (const struct timeval *tv, struct gc_arena *gc);
 
-extern volatile time_t now; /* updated frequently to time(NULL) */
+extern time_t now; /* updated frequently to time(NULL) */
+
+void time_test (void);
+
+#if TIME_BACKTRACK_PROTECTION && defined(HAVE_GETTIMEOFDAY)
+
+void update_now (const time_t system_time);
+
+extern time_t now_usec;
+void update_now_usec (struct timeval *tv);
+
+static inline int
+openvpn_gettimeofday (struct timeval *tv, void *tz)
+{
+  const int status = gettimeofday (tv, tz);
+  if (!status)
+    {
+      update_now_usec (tv);
+      tv->tv_sec = now;
+      tv->tv_usec = now_usec;
+    }
+  return status;
+}
 
 static inline void
 update_time (void)
 {
+#ifdef WIN32
+  /* on WIN32, gettimeofday is faster than time(NULL) */
+  struct timeval tv;
+  openvpn_gettimeofday (&tv, NULL);
+#else
+  update_now (time (NULL));
+#endif
+}
+
+#else /* !(TIME_BACKTRACK_PROTECTION && defined(HAVE_GETTIMEOFDAY)) */
+
+static inline void
+update_time (void)
+{
+#if defined(WIN32) && defined(HAVE_GETTIMEOFDAY)
+  struct timeval tv;
+  if (!gettimeofday (&tv, NULL))
+    {
+      if (tv.tv_sec != now)
+	now = tv.tv_sec;
+    }
+#else
   const time_t real_time = time (NULL);
   if (real_time != now)
     now = real_time;
+#endif
+}
+
+#ifdef HAVE_GETTIMEOFDAY
+
+static inline int
+openvpn_gettimeofday (struct timeval *tv, void *tz)
+{
+  return gettimeofday (tv, tz);
+}
+
+#endif
+
+#endif /* TIME_BACKTRACK_PROTECTION && defined(HAVE_GETTIMEOFDAY) */
+
+static inline time_t
+openvpn_time (time_t *t)
+{
+  update_time ();
+  if (t)
+    *t = now;
+  return now;
 }
 
 static inline void
