@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -37,7 +37,7 @@ check_ping_restart (struct context *c)
       && event_timeout_trigger (&c->c2.ping_rec_interval,
 				&c->c2.timeval,
 				(!c->options.ping_timer_remote
-				 || addr_defined (&c->c1.link_socket_addr.actual))
+				 || link_socket_actual_defined (&c->c1.link_socket_addr.actual))
 				? ETT_DEFAULT : 15))
     check_ping_restart_dowork (c);
 }
