@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -48,7 +48,7 @@
 
 #include "openvpn-plugin.h"
 
-#define DEBUG(verb) ((verb) >= 7)
+#define DEBUG(verb) ((verb) >= 4)
 
 /* Command codes for foreground -> background communication */
 #define COMMAND_VERIFY 0
@@ -206,6 +206,8 @@ send_string (int fd, const char *string)
     return -1;
 }
 
+#ifdef DO_DAEMONIZE
+
 /*
  * Daemonize if "daemon" env var is true.
  * Preserve stderr across daemonization if
@@ -233,6 +235,8 @@ daemonize (const char *envp[])
     }
 }
 
+#endif
+
 /*
  * Close most of parent's fds.
  * Keep stdin/stdout/stderr, plus one
@@ -301,6 +305,8 @@ openvpn_plugin_open_v1 (unsigned int *type_mask, const char *argv[], const char
    * Allocate our context
    */
   context = (struct auth_pam_context *) calloc (1, sizeof (struct auth_pam_context));
+  if (!context)
+    goto error;
   context->foreground_fd = -1;
 
   /*
@@ -405,8 +411,10 @@ openvpn_plugin_open_v1 (unsigned int *type_mask, const char *argv[], const char
       /* Ignore most signals (the parent will receive them) */
       set_signals ();
 
+#ifdef DO_DAEMONIZE
       /* Daemonize if --daemon option is set. */
       daemonize (envp);
+#endif
 
       /* execute the event loop */
       pam_server (fd[1], argv[1], context->verb, &name_value_list);
@@ -486,7 +494,7 @@ openvpn_plugin_abort_v1 (openvpn_plugin_handle_t handle)
   struct auth_pam_context *context = (struct auth_pam_context *) handle;
 
   /* tell background process to exit */
-  if (context->foreground_fd >= 0)
+  if (context && context->foreground_fd >= 0)
     {
       send_control (context->foreground_fd, COMMAND_EXIT);
       close (context->foreground_fd);
@@ -709,8 +717,14 @@ pam_server (int fd, const char *service, int verb, const struct name_value_list
 	    }
 
 	  if (DEBUG (verb))
-	    fprintf (stderr, "AUTH-PAM: BACKGROUND: USER/PASS: %s/%s\n",
-		     up.username, up.password);
+	    {
+#if 0
+	      fprintf (stderr, "AUTH-PAM: BACKGROUND: USER/PASS: %s/%s\n",
+		       up.username, up.password);
+#else
+	      fprintf (stderr, "AUTH-PAM: BACKGROUND: USER: %s\n", up.username);
+#endif
+	    }
 
 	  if (pam_auth (service, &up)) /* Succeeded */
 	    {
