@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -25,8 +25,11 @@
 #ifndef PROTO_H
 #define PROTO_H
 
+#include "common.h"
 #include "buffer.h"
 
+#pragma pack(1)
+
 /*
  * Tunnel types
  */
@@ -35,6 +38,13 @@
 #define DEV_TYPE_TUN   2    /* point-to-point IP tunnel */
 #define DEV_TYPE_TAP   3    /* ethernet (802.3) tunnel */
 
+/* TUN topologies */
+
+#define TOP_UNDEF   0
+#define TOP_NET30   1
+#define TOP_P2P     2
+#define TOP_SUBNET  3
+
 /*
  * IP and Ethernet protocol structs.  For portability,
  * OpenVPN needs its own definitions of these structs, and
@@ -54,6 +64,24 @@ struct openvpn_ethhdr
   uint16_t proto;                     /* packet type ID field */
 };
 
+struct openvpn_arp {
+# define ARP_MAC_ADDR_TYPE 0x0001
+  uint16_t mac_addr_type;       /* 0x0001 */
+
+  uint16_t proto_addr_type;     /* 0x0800 */
+  uint8_t  mac_addr_size;       /* 0x06 */
+  uint8_t  proto_addr_size;     /* 0x04 */
+
+# define ARP_REQUEST 0x0001
+# define ARP_REPLY   0x0002
+  uint16_t arp_command;         /* 0x0001 for ARP request, 0x0002 for ARP reply */
+
+  uint8_t   mac_src[OPENVPN_ETH_ALEN];
+  in_addr_t ip_src;
+  uint8_t   mac_dest[OPENVPN_ETH_ALEN];
+  in_addr_t ip_dest;
+};
+
 struct openvpn_iphdr {
 # define OPENVPN_IPH_GET_VER(v) (((v) >> 4) & 0x0F)
 # define OPENVPN_IPH_GET_LEN(v) (((v) & 0x0F) << 2)
@@ -121,6 +149,8 @@ struct openvpn_tcphdr {
 #define	OPENVPN_TCPOPT_MAXSEG  2
 #define OPENVPN_TCPOLEN_MAXSEG 4
 
+#pragma pack()
+
 /*
  * The following macro is used to update an
  * internet checksum.  "acc" is a 32-bit
@@ -160,4 +190,13 @@ struct openvpn_tcphdr {
  */
 bool is_ipv4 (int tunnel_type, struct buffer *buf);
 
+#ifdef PACKET_TRUNCATION_CHECK
+void ipv4_packet_size_verify (const uint8_t *data,
+			      const int size,
+			      const int tunnel_type,
+			      const char
+			      *prefix,
+			      counter_type *errors);
+#endif
+
 #endif
