@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -25,22 +25,48 @@
 #ifndef PROXY_H
 #define PROXY_H
 
-#ifdef ENABLE_HTTP_PROXY
-
 #include "buffer.h"
 #include "misc.h"
 
+#ifdef GENERAL_PROXY_SUPPORT
+
+/*
+ * Return value for get_proxy_settings to automatically
+ * determine proxy information.
+ */
+struct auto_proxy_info_entry {
+  char *server;
+  int port;
+};
+
+struct auto_proxy_info {
+  struct auto_proxy_info_entry http;
+  struct auto_proxy_info_entry socks;
+};
+
+struct auto_proxy_info *get_proxy_settings (char **err, struct gc_arena *gc);
+
+#ifdef WIN32
+void show_win_proxy_settings (const int msglevel);
+#endif /* WIN32 */
+
+#endif /* GENERAL_PROXY_SUPPORT */
+
+#ifdef ENABLE_HTTP_PROXY
+
 /* HTTP CONNECT authentication methods */
 #define HTTP_AUTH_NONE  0
 #define HTTP_AUTH_BASIC 1
 #define HTTP_AUTH_NTLM  2
 #define HTTP_AUTH_N     3
+#define HTTP_AUTH_NTLM2 4
 
 struct http_proxy_options {
   const char *server;
   int port;
   bool retry;
   int timeout;
+  bool auth_retry;
   const char *auth_method_string;
   const char *auth_file;
   const char *http_version;
@@ -54,10 +80,12 @@ struct http_proxy_info {
   struct user_pass up;
 };
 
-struct http_proxy_info *new_http_proxy (const struct http_proxy_options *o,
-					struct gc_arena *gc);
+struct http_proxy_info *http_proxy_new (const struct http_proxy_options *o,
+					struct auto_proxy_info *auto_proxy_info);
 
-void establish_http_proxy_passthru (struct http_proxy_info *p,
+void http_proxy_close (struct http_proxy_info *hp);
+
+bool establish_http_proxy_passthru (struct http_proxy_info *p,
 				    socket_descriptor_t sd, /* already open to proxy */
 				    const char *host,       /* openvpn server remote */
 				    const int port,         /* openvpn server port */
@@ -67,5 +95,6 @@ void establish_http_proxy_passthru (struct http_proxy_info *p,
 uint8_t *make_base64_string2 (const uint8_t *str, int str_len, struct gc_arena *gc);
 uint8_t *make_base64_string (const uint8_t *str, struct gc_arena *gc);
 
-#endif
-#endif
+#endif /* ENABLE_HTTP_PROXY */
+
+#endif /* PROXY_H */
