@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -49,9 +49,11 @@ int parse_signal (const char *signame);
 const char *signal_name (const int sig, const bool upper);
 const char *signal_description (const int signum, const char *sigtext);
 void throw_signal (const int signum);
+void throw_signal_soft (const int signum, const char *signal_text);
 
 void pre_init_signal_catch (void);
 void post_init_signal_catch (void);
+void restore_signal_state (void);
 
 void print_signal (const struct signal_info *si, const char *title, int msglevel);
 void print_status (const struct context *c, struct status_output *so);
