@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -29,16 +29,10 @@
  * see RFC 1928, only supports "no authentication"
  */
 
-#ifdef WIN32
-#include "config-win32.h"
-#else
-#include "config.h"
-#endif
+#include "syshead.h"
 
 #ifdef ENABLE_SOCKS
 
-#include "syshead.h"
-
 #include "common.h"
 #include "misc.h"
 #include "win32.h"
@@ -57,13 +51,27 @@ socks_adjust_frame_parameters (struct frame *frame, int proto)
 }
 
 struct socks_proxy_info *
-new_socks_proxy (const char *server,
+socks_proxy_new (const char *server,
 		 int port,
 		 bool retry,
-		 struct gc_arena *gc)
+		 struct auto_proxy_info *auto_proxy_info)
 {
   struct socks_proxy_info *p;
-  ALLOC_OBJ_CLEAR_GC (p, struct socks_proxy_info, gc);
+
+  if (auto_proxy_info)
+    {
+      if (!server)
+	{
+	  if (!auto_proxy_info->socks.server)
+	    return NULL;
+
+	  server = auto_proxy_info->socks.server;
+	  port = auto_proxy_info->socks.port;
+	}
+    }
+
+  ALLOC_OBJ_CLEAR (p, struct socks_proxy_info);
+
   ASSERT (server);
   ASSERT (legal_ipv4_port (port));
 
@@ -75,6 +83,12 @@ new_socks_proxy (const char *server,
   return p;
 }
 
+void
+socks_proxy_close (struct socks_proxy_info *sp)
+{
+  free (sp);
+}
+
 static bool
 socks_handshake (socket_descriptor_t sd, volatile int *signal_received)
 {
@@ -148,7 +162,8 @@ socks_handshake (socket_descriptor_t sd, volatile int *signal_received)
 }
 
 static bool
-recv_socks_reply (socket_descriptor_t sd, struct sockaddr_in *addr,
+recv_socks_reply (socket_descriptor_t sd,
+		  struct openvpn_sockaddr *addr,
 		  volatile int *signal_received)
 {
   char atyp = '\0';
@@ -159,9 +174,9 @@ recv_socks_reply (socket_descriptor_t sd, struct sockaddr_in *addr,
 
   if (addr != NULL)
     {
-      addr->sin_family = AF_INET;
-      addr->sin_addr.s_addr = htonl (INADDR_ANY);
-      addr->sin_port = htons (0);
+      addr->sa.sin_family = AF_INET;
+      addr->sa.sin_addr.s_addr = htonl (INADDR_ANY);
+      addr->sa.sin_port = htons (0);
     }
 
   while (len < 4 + alen + 2)
@@ -248,8 +263,8 @@ recv_socks_reply (socket_descriptor_t sd, struct sockaddr_in *addr,
   /* ATYP == 1 (IP V4 address) */
   if (atyp == '\x01' && addr != NULL)
     {
-      memcpy (&addr->sin_addr, buf + 4, sizeof (addr->sin_addr));
-      memcpy (&addr->sin_port, buf + 8, sizeof (addr->sin_port));
+      memcpy (&addr->sa.sin_addr, buf + 4, sizeof (addr->sa.sin_addr));
+      memcpy (&addr->sa.sin_port, buf + 8, sizeof (addr->sa.sin_port));
     }
 
 
@@ -310,7 +325,7 @@ void
 establish_socks_proxy_udpassoc (struct socks_proxy_info *p,
 			        socket_descriptor_t ctrl_sd, /* already open to proxy */
 				socket_descriptor_t udp_sd,
-				struct sockaddr_in *relay_addr,
+				struct openvpn_sockaddr *relay_addr,
 			        volatile int *signal_received)
 {
   if (!socks_handshake (ctrl_sd, signal_received))
@@ -352,7 +367,7 @@ establish_socks_proxy_udpassoc (struct socks_proxy_info *p,
  */
 void
 socks_process_incoming_udp (struct buffer *buf,
-			    struct sockaddr_in *from)
+			    struct link_socket_actual *from)
 {
   int atyp;
 
@@ -367,8 +382,8 @@ socks_process_incoming_udp (struct buffer *buf,
   if (atyp != 1)		/* ATYP == 1 (IP V4) */
     goto error;
 
-  buf_read (buf, &from->sin_addr, sizeof (from->sin_addr));
-  buf_read (buf, &from->sin_port, sizeof (from->sin_port));
+  buf_read (buf, &from->dest.sa.sin_addr, sizeof (from->dest.sa.sin_addr));
+  buf_read (buf, &from->dest.sa.sin_port, sizeof (from->dest.sa.sin_port));
 
   return;
 
@@ -385,7 +400,7 @@ socks_process_incoming_udp (struct buffer *buf,
  */
 int
 socks_process_outgoing_udp (struct buffer *buf,
-			    struct sockaddr_in *to)
+			    const struct link_socket_actual *to)
 {
   /* 
    * Get a 10 byte subset buffer prepended to buf --
@@ -400,8 +415,8 @@ socks_process_outgoing_udp (struct buffer *buf,
   buf_write_u16 (&head, 0);	/* RSV = 0 */
   buf_write_u8 (&head, 0);	/* FRAG = 0 */
   buf_write_u8 (&head, '\x01'); /* ATYP = 1 (IP V4) */
-  buf_write (&head, &to->sin_addr, sizeof (to->sin_addr));
-  buf_write (&head, &to->sin_port, sizeof (to->sin_port));
+  buf_write (&head, &to->dest.sa.sin_addr, sizeof (to->dest.sa.sin_addr));
+  buf_write (&head, &to->dest.sa.sin_port, sizeof (to->dest.sa.sin_port));
 
   return 10;
 }
