@@ -1,14 +1,10 @@
 /*
- *  TAP-Win32 -- A kernel driver to provide virtual tap device
- *               functionality on Windows.  Originally derived
- *               from the CIPE-Win32 project by Damion K. Wilson,
- *               with extensive modifications by James Yonan.
+ *  TAP-Win32/TAP-Win64 -- A kernel driver to provide virtual tap
+ *                         device functionality on Windows.
  *
- *  All source code which derives from the CIPE-Win32 project is
- *  Copyright (C) Damion K. Wilson, 2003, and is released under the
- *  GPL version 2 (see below).
+ *  This code was inspired by the CIPE-Win32 driver by Damion K. Wilson.
  *
- *  All other source code is Copyright (C) 2002-2005 OpenVPN Solutions LLC,
+ *  This source code is Copyright (C) 2002-2009 OpenVPN Technologies, Inc.,
  *  and is released under the GPL version 2 (see below).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -35,9 +31,15 @@
 // By default we operate as a "tap" virtual ethernet
 // 802.3 interface, but we can emulate a "tun"
 // interface (point-to-point IPv4) through the
-// TAP_IOCTL_CONFIG_POINT_TO_POINT ioctl.
+// TAP_IOCTL_CONFIG_POINT_TO_POINT or
+// TAP_IOCTL_CONFIG_TUN ioctl.
 //======================================================
 
+#include "../../autodefs/defs.h"
+#ifndef DDKVER_MAJOR
+#error DDKVER_MAJOR must be defined as the major number of the DDK Version
+#endif
+
 #define NDIS_MINIPORT_DRIVER
 #define BINARY_COMPATIBLE 0
 #define NDIS50_MINIPORT 1
@@ -46,11 +48,33 @@
 #define NTSTRSAFE_LIB
 
 // Debug info output
-#define ALSO_DBGPRINT      1
-#define DEBUGP_AT_DISPATCH 0
+#define ALSO_DBGPRINT           1
+#define DEBUGP_AT_DISPATCH      0
 
+//========================================================
+// Check for truncated IPv4 packets, log errors if found.
+//========================================================
+#define PACKET_TRUNCATION_CHECK 0
+
+//========================================================
+// EXPERIMENTAL -- Configure TAP device object to be
+// accessible from non-administrative accounts, based
+// on an advanced properties setting.
+//
+// Duplicates the functionality of OpenVPN's
+// --allow-nonadmin directive.
+//========================================================
+#define ENABLE_NONADMIN 1
+
+#if DDKVER_MAJOR < 5600
 #include <ndis.h>
 #include <ntstrsafe.h>
+#include <ntddk.h>
+#else
+#include <ntifs.h>
+#include <ndis.h>
+#include <ntstrsafe.h>
+#endif
 
 #include "lock.h"
 #include "constants.h"
@@ -260,12 +284,12 @@ TapDriverUnload (IN PDRIVER_OBJECT p_DriverObject)
 //                            Adapter Initialization
 //==========================================================
 NDIS_STATUS AdapterCreate
-  (OUT PNDIS_STATUS p_ErrorStatus,
-   OUT PUINT p_MediaIndex,
-   IN PNDIS_MEDIUM p_Media,
-   IN UINT p_MediaCount,
-   IN NDIS_HANDLE p_AdapterHandle,
-   IN NDIS_HANDLE p_ConfigurationHandle)
+(OUT PNDIS_STATUS p_ErrorStatus,
+ OUT PUINT p_MediaIndex,
+ IN PNDIS_MEDIUM p_Media,
+ IN UINT p_MediaCount,
+ IN NDIS_HANDLE p_AdapterHandle,
+ IN NDIS_HANDLE p_ConfigurationHandle)
 {
   TapAdapterPointer l_Adapter = NULL;
 
@@ -274,6 +298,12 @@ NDIS_STATUS AdapterCreate
   UINT l_Index;
   NDIS_STATUS status;
 
+#if ENABLE_NONADMIN
+  BOOLEAN enable_non_admin = FALSE;
+#endif
+
+  DEBUGP (("[TAP] AdapterCreate called\n"));
+
   //====================================
   // Make sure adapter type is supported
   //====================================
@@ -343,19 +373,6 @@ NDIS_STATUS AdapterCreate
 				       AdapterHalt);
   l_Adapter->m_RegisteredAdapterShutdownHandler = TRUE;
 
-  //====================================
-  // Allocate and construct adapter name
-  //====================================
-
-  if (RtlUnicodeStringToAnsiString (
-       &l_Adapter->m_NameAnsi,
-       &((PNDIS_MINIPORT_BLOCK) p_AdapterHandle)->MiniportName,
-       TRUE) != STATUS_SUCCESS)
-    {
-      AdapterFreeResources (l_Adapter);
-      return NDIS_STATUS_RESOURCES;
-    }
-
   //============================================
   // Get parameters from registry which were set
   // in the adapter advanced properties dialog.
@@ -371,67 +388,153 @@ NDIS_STATUS AdapterCreate
     l_Adapter->m_MediaState = FALSE;
 
     NdisOpenConfiguration (&status, &configHandle, p_ConfigurationHandle);
-    if (status == NDIS_STATUS_SUCCESS)
+    if (status != NDIS_STATUS_SUCCESS)
       {
-	/* Read MTU setting from registry */
-	{
-	  NDIS_STRING key = NDIS_STRING_CONST("MTU");
-	  NdisReadConfiguration (&status, &parm, configHandle,
-				 &key, NdisParameterInteger);
-	  if (status == NDIS_STATUS_SUCCESS)
-	    {
-	      if (parm->ParameterType == NdisParameterInteger)
-		{
-		  int mtu = parm->ParameterData.IntegerData;
-		  if (mtu < MINIMUM_MTU)
-		    mtu = MINIMUM_MTU;
-		  if (mtu > MAXIMUM_MTU)
-		    mtu = MAXIMUM_MTU;
-		  l_Adapter->m_MTU = mtu;
-		}
-	    }
-	}
-
-	/* Read Media Status setting from registry */
-	{
-	  NDIS_STRING key = NDIS_STRING_CONST("MediaStatus");
-	  NdisReadConfiguration (&status, &parm, configHandle,
-				 &key, NdisParameterInteger);
-	  if (status == NDIS_STATUS_SUCCESS)
-	    {
-	      if (parm->ParameterType == NdisParameterInteger)
-		{
-		  if (parm->ParameterData.IntegerData)
-		    {
-		      l_Adapter->m_MediaStateAlwaysConnected = TRUE;
-		      l_Adapter->m_MediaState = TRUE;
-		    }
-		}
-	    }
-	}
-
-	/* Read optional MAC setting from registry */
-	{
-	  NDIS_STRING key = NDIS_STRING_CONST("MAC");
-	  ANSI_STRING mac_string;
-	  NdisReadConfiguration (&status, &parm, configHandle,
-				 &key, NdisParameterString);
-	  if (status == NDIS_STATUS_SUCCESS)
-	    {
-	      if (parm->ParameterType == NdisParameterString)
-		{
-		  if (RtlUnicodeStringToAnsiString (&mac_string, &parm->ParameterData.StringData, TRUE) == STATUS_SUCCESS)
-		    {
-		      l_MacFromRegistry = ParseMAC (l_Adapter->m_MAC, mac_string.Buffer);
-		      RtlFreeAnsiString (&mac_string);
-		    }
-		}
-	    }
-	}
-
-	NdisCloseConfiguration (configHandle);
+	DEBUGP (("[TAP] Couldn't open adapter registry\n"));
+	AdapterFreeResources (l_Adapter);
+	return status;
       }
 
+    //====================================
+    // Allocate and construct adapter name
+    //====================================
+    {
+      
+      NDIS_STRING mkey = NDIS_STRING_CONST("MiniportName");
+      NDIS_STRING vkey = NDIS_STRING_CONST("NdisVersion");
+      NDIS_STATUS vstatus;
+      NDIS_CONFIGURATION_PARAMETER *vparm;
+
+      NdisReadConfiguration (&vstatus, &vparm, configHandle, &vkey, NdisParameterInteger);
+      if (vstatus == NDIS_STATUS_SUCCESS)
+	DEBUGP (("[TAP] NdisReadConfiguration NdisVersion=%X\n", vparm->ParameterData.IntegerData));
+
+      NdisReadConfiguration (&status, &parm, configHandle, &mkey, NdisParameterString);
+      if (status == NDIS_STATUS_SUCCESS)
+	{
+	  if (parm->ParameterType == NdisParameterString)
+	    {
+	      DEBUGP (("[TAP] NdisReadConfiguration (MiniportName=%S)\n", parm->ParameterData.StringData.Buffer));
+
+	      if (RtlUnicodeStringToAnsiString (
+						&l_Adapter->m_NameAnsi,
+						&parm->ParameterData.StringData,
+						TRUE) != STATUS_SUCCESS)
+		{
+		  DEBUGP (("[TAP] MiniportName failed\n"));
+		  status = NDIS_STATUS_RESOURCES;
+		}
+	    }
+	}
+      else
+	{
+	  /* "MiniportName" is available only XP and above.  Not on Windows 2000. */
+	  if (vstatus == NDIS_STATUS_SUCCESS && vparm->ParameterData.IntegerData == 0x50000)
+	    {
+	      /* Fallback for Windows 2000 with NDIS version 5.00.00
+		 Don't use this on Vista, 'NDIS_MINIPORT_BLOCK' was changed! */
+	      if (RtlUnicodeStringToAnsiString (&l_Adapter->m_NameAnsi,
+						&((struct WIN2K_NDIS_MINIPORT_BLOCK *) p_AdapterHandle)->MiniportName,
+						TRUE) != STATUS_SUCCESS)
+		{
+		  DEBUGP (("[TAP] MiniportName (W2K) failed\n"));
+		  status = NDIS_STATUS_RESOURCES;
+		}
+	      else
+		{
+		  DEBUGP (("[TAP] MiniportName (W2K) succeeded: %s\n", l_Adapter->m_NameAnsi.Buffer));
+		  status = NDIS_STATUS_SUCCESS;
+		}
+	    }
+	}
+    }
+
+    /* Can't continue without name (see macro 'NAME') */
+    if (status != NDIS_STATUS_SUCCESS || !l_Adapter->m_NameAnsi.Buffer)
+      {
+	NdisCloseConfiguration (configHandle);
+	AdapterFreeResources (l_Adapter);
+	DEBUGP (("[TAP] failed to get miniport name\n"));
+	return NDIS_STATUS_RESOURCES;
+      }
+
+    /* Read MTU setting from registry */
+    {
+      NDIS_STRING key = NDIS_STRING_CONST("MTU");
+      NdisReadConfiguration (&status, &parm, configHandle,
+			     &key, NdisParameterInteger);
+      if (status == NDIS_STATUS_SUCCESS)
+	{
+	  if (parm->ParameterType == NdisParameterInteger)
+	    {
+	      int mtu = parm->ParameterData.IntegerData;
+	      if (mtu < MINIMUM_MTU)
+		mtu = MINIMUM_MTU;
+	      if (mtu > MAXIMUM_MTU)
+		mtu = MAXIMUM_MTU;
+	      l_Adapter->m_MTU = mtu;
+	    }
+	}
+    }
+
+    /* Read Media Status setting from registry */
+    {
+      NDIS_STRING key = NDIS_STRING_CONST("MediaStatus");
+      NdisReadConfiguration (&status, &parm, configHandle,
+			     &key, NdisParameterInteger);
+      if (status == NDIS_STATUS_SUCCESS)
+	{
+	  if (parm->ParameterType == NdisParameterInteger)
+	    {
+	      if (parm->ParameterData.IntegerData)
+		{
+		  l_Adapter->m_MediaStateAlwaysConnected = TRUE;
+		  l_Adapter->m_MediaState = TRUE;
+		}
+	    }
+	}
+    }
+
+#if ENABLE_NONADMIN
+    /* Read AllowNonAdmin setting from registry */
+    {
+      NDIS_STRING key = NDIS_STRING_CONST("AllowNonAdmin");
+      NdisReadConfiguration (&status, &parm, configHandle,
+			     &key, NdisParameterInteger);
+      if (status == NDIS_STATUS_SUCCESS)
+	{
+	  if (parm->ParameterType == NdisParameterInteger)
+	    {
+	      if (parm->ParameterData.IntegerData)
+		{
+		  enable_non_admin = TRUE;
+		}
+	    }
+	}
+    }
+#endif
+
+    /* Read optional MAC setting from registry */
+    {
+      NDIS_STRING key = NDIS_STRING_CONST("MAC");
+      ANSI_STRING mac_string;
+      NdisReadConfiguration (&status, &parm, configHandle,
+			     &key, NdisParameterString);
+      if (status == NDIS_STATUS_SUCCESS)
+	{
+	  if (parm->ParameterType == NdisParameterString)
+	    {
+	      if (RtlUnicodeStringToAnsiString (&mac_string, &parm->ParameterData.StringData, TRUE) == STATUS_SUCCESS)
+		{
+		  l_MacFromRegistry = ParseMAC (l_Adapter->m_MAC, mac_string.Buffer);
+		  RtlFreeAnsiString (&mac_string);
+		}
+	    }
+	}
+    }
+
+    NdisCloseConfiguration (configHandle);
+
     DEBUGP (("[%s] MTU=%d\n", NAME (l_Adapter), l_Adapter->m_MTU));
   }
 
@@ -443,9 +546,9 @@ NDIS_STATUS AdapterCreate
     GenerateRandomMac (l_Adapter->m_MAC, NAME (l_Adapter));
 
   DEBUGP (("[%s] Using MAC %x:%x:%x:%x:%x:%x\n",
-	    NAME (l_Adapter),
-	    l_Adapter->m_MAC[0], l_Adapter->m_MAC[1], l_Adapter->m_MAC[2],
-	    l_Adapter->m_MAC[3], l_Adapter->m_MAC[4], l_Adapter->m_MAC[5]));
+	   NAME (l_Adapter),
+	   l_Adapter->m_MAC[0], l_Adapter->m_MAC[1], l_Adapter->m_MAC[2],
+	   l_Adapter->m_MAC[3], l_Adapter->m_MAC[4], l_Adapter->m_MAC[5]));
 
   //==================
   // Set broadcast MAC
@@ -465,6 +568,7 @@ NDIS_STATUS AdapterCreate
     if (tap_status != NDIS_STATUS_SUCCESS)
       {
 	AdapterFreeResources (l_Adapter);
+	DEBUGP (("[TAP] CreateTapDevice failed\n"));
 	return tap_status;
       }
   }
@@ -474,11 +578,17 @@ NDIS_STATUS AdapterCreate
       NOTE_ERROR ();
       TapDeviceFreeResources (&l_Adapter->m_Extension);
       AdapterFreeResources (l_Adapter);
+      DEBUGP (("[TAP] AddAdapterToInstanceList failed\n"));
       return NDIS_STATUS_RESOURCES;
     }
 
   l_Adapter->m_InterfaceIsRunning = TRUE;
 
+#if ENABLE_NONADMIN
+  if (enable_non_admin)
+    AllowNonAdmin (&l_Adapter->m_Extension);
+#endif
+
   return NDIS_STATUS_SUCCESS;
 }
 
@@ -524,7 +634,7 @@ AdapterFreeResources (TapAdapterPointer p_Adapter)
     NdisMDeregisterAdapterShutdownHandler (p_Adapter->m_MiniportAdapterHandle);
 
   if (p_Adapter->m_MCLockAllocated)
-    NdisFreeSpinLock (&l_Adapter->m_MCLock);
+    NdisFreeSpinLock (&p_Adapter->m_MCLock);
 }
 
 VOID
@@ -578,6 +688,8 @@ TapDeviceFreeResources (TapExtensionPointer p_Extension)
     QueueFree (p_Extension->m_PacketQueue);
   if (p_Extension->m_IrpQueue)
     QueueFree (p_Extension->m_IrpQueue);
+  if (p_Extension->m_InjectQueue)
+    QueueFree (p_Extension->m_InjectQueue);
 
   if (p_Extension->m_CreatedUnicodeLinkName)
     RtlFreeUnicodeString (&p_Extension->m_UnicodeLinkName);
@@ -603,8 +715,14 @@ TapDeviceFreeResources (TapExtensionPointer p_Extension)
   if (p_Extension->m_TapName)
     MemFree (p_Extension->m_TapName, NAME_BUFFER_SIZE);
   
+  if (p_Extension->m_InjectDpcInitialized)
+    KeRemoveQueueDpc (&p_Extension->m_InjectDpc);
+
   if (p_Extension->m_AllocatedSpinlocks)
-    NdisFreeSpinLock (&p_Extension->m_QueueLock);
+    {
+      NdisFreeSpinLock (&p_Extension->m_QueueLock);
+      NdisFreeSpinLock (&p_Extension->m_InjectLock);
+    }
 }
 
 //========================================================================
@@ -818,19 +936,28 @@ CreateTapDevice (TapExtensionPointer p_Extension, const char *p_Name)
   //========================================================
 
   NdisAllocateSpinLock (&p_Extension->m_QueueLock);
+  NdisAllocateSpinLock (&p_Extension->m_InjectLock);
   p_Extension->m_AllocatedSpinlocks = TRUE;
 
   p_Extension->m_PacketQueue = QueueInit (PACKET_QUEUE_SIZE);
   p_Extension->m_IrpQueue = QueueInit (IRP_QUEUE_SIZE);
-
+  p_Extension->m_InjectQueue = QueueInit (INJECT_QUEUE_SIZE);
   if (!p_Extension->m_PacketQueue
-      || !p_Extension->m_IrpQueue)
+      || !p_Extension->m_IrpQueue
+      || !p_Extension->m_InjectQueue)
     {
       DEBUGP (("[%s] couldn't alloc TAP queues\n", p_Name));
       l_Return = NDIS_STATUS_RESOURCES;
       goto cleanup;
     }
 
+  //=================================================================
+  // Initialize deferred procedure call for DHCP/ARP packet injection
+  //=================================================================
+
+  KeInitializeDpc (&p_Extension->m_InjectDpc, InjectPacketDpc, NULL);
+  p_Extension->m_InjectDpcInitialized = TRUE;
+
   //========================
   // Finalize initialization
   //========================
@@ -1011,7 +1138,7 @@ NDIS_STATUS AdapterQuery
       break;
       
     case OID_GEN_LINK_SPEED:
-      l_Query.m_Long = 100000;
+      l_Query.m_Long = 100000; // rate / 100 bps
       break;
 
     case OID_802_3_PERMANENT_ADDRESS:
@@ -1311,7 +1438,8 @@ AdapterTransmit (IN NDIS_HANDLE p_AdapterContext,
 		 IN UINT p_Flags)
 {
   TapAdapterPointer l_Adapter = (TapAdapterPointer) p_AdapterContext;
-  ULONG l_Index = 0, l_BufferLength = 0, l_PacketLength = 0;
+  ULONG l_Index = 0, l_PacketLength = 0;
+  UINT l_BufferLength = 0;
   PIRP l_IRP;
   TapPacketPointer l_PacketBuffer;
   PNDIS_BUFFER l_NDIS_Buffer;
@@ -1348,18 +1476,39 @@ AdapterTransmit (IN NDIS_HANDLE p_AdapterContext,
 
   __try
   {
-    for (l_Index = 0; l_NDIS_Buffer && l_Index < l_PacketLength;
-	 l_Index += l_BufferLength)
+    l_Index = 0;
+    while (l_NDIS_Buffer && l_Index < l_PacketLength)
       {
+	ULONG newlen;
 	NdisQueryBuffer (l_NDIS_Buffer, (PVOID *) & l_Buffer,
 			 &l_BufferLength);
+	newlen = l_Index + l_BufferLength;
+	if (newlen > l_PacketLength)
+	  {
+	    NOTE_ERROR ();
+	    goto no_queue; /* overflow */
+	  }
 	NdisMoveMemory (l_PacketBuffer->m_Data + l_Index, l_Buffer,
 			l_BufferLength);
+	l_Index = newlen;
 	NdisGetNextBuffer (l_NDIS_Buffer, &l_NDIS_Buffer);
       }
+    if (l_Index != l_PacketLength)
+      {
+	NOTE_ERROR ();
+	goto no_queue; /* underflow */
+      }
 
     DUMP_PACKET ("AdapterTransmit", l_PacketBuffer->m_Data, l_PacketLength);
 
+    //=====================================================
+    // If IPv4 packet, check whether or not packet
+    // was truncated.
+    //=====================================================
+#if PACKET_TRUNCATION_CHECK
+    IPv4PacketSizeVerify (l_PacketBuffer->m_Data, l_PacketLength, FALSE, "TX", &l_Adapter->m_TxTrunc);
+#endif
+
     //=====================================================
     // Are we running in DHCP server masquerade mode?
     //
@@ -1381,6 +1530,7 @@ AdapterTransmit (IN NDIS_HANDLE p_AdapterContext,
 			    (PARP_PACKET) l_PacketBuffer->m_Data,
 			    l_Adapter->m_dhcp_addr,
 			    l_Adapter->m_dhcp_server_ip,
+			    ~0,
 			    l_Adapter->m_dhcp_server_mac))
 	      goto no_queue;
 	  }
@@ -1417,7 +1567,7 @@ AdapterTransmit (IN NDIS_HANDLE p_AdapterContext,
     // In Point-To-Point mode, check to see whether
     // packet is ARP or IPv4 (if neither, then drop).
     //===============================================
-    if (l_Adapter->m_PointToPoint)
+    if (l_Adapter->m_tun)
       {
 	ETH_HEADER *e;
 
@@ -1438,7 +1588,8 @@ AdapterTransmit (IN NDIS_HANDLE p_AdapterContext,
 	    ProcessARP (l_Adapter,
 			(PARP_PACKET) l_PacketBuffer->m_Data,
 			l_Adapter->m_localIP,
-			l_Adapter->m_remoteIP,
+			l_Adapter->m_remoteNetwork,
+			l_Adapter->m_remoteNetmask,
 			l_Adapter->m_TapToUser.dest);
 
 	  default:
@@ -1458,7 +1609,7 @@ AdapterTransmit (IN NDIS_HANDLE p_AdapterContext,
 	      goto no_queue;
 
 	    // Packet looks like IPv4, queue it.
-	    l_PacketBuffer->m_SizeFlags |= TP_POINT_TO_POINT;
+	    l_PacketBuffer->m_SizeFlags |= TP_TUN;
 	  }
       }
 
@@ -1669,21 +1820,34 @@ TapDeviceHook (IN PDEVICE_OBJECT p_DeviceObject, IN PIRP p_IRP)
 		NULL,
 		NULL,
 		STRSAFE_FILL_BEHIND_NULL | STRSAFE_IGNORE_NULLS,
-		"State=%s Err=[%s/%d] #O=%d Tx=[%d,%d] Rx=[%d,%d] IrpQ=[%d,%d,%d] PktQ=[%d,%d,%d]",
+#if PACKET_TRUNCATION_CHECK
+		"State=%s Err=[%s/%d] #O=%d Tx=[%d,%d,%d] Rx=[%d,%d,%d] IrpQ=[%d,%d,%d] PktQ=[%d,%d,%d] InjQ=[%d,%d,%d]",
+#else
+		"State=%s Err=[%s/%d] #O=%d Tx=[%d,%d] Rx=[%d,%d] IrpQ=[%d,%d,%d] PktQ=[%d,%d,%d] InjQ=[%d,%d,%d]",
+#endif
 		state,
 		g_LastErrorFilename,
 		g_LastErrorLineNumber,
 		(int)l_Adapter->m_Extension.m_NumTapOpens,
 		(int)l_Adapter->m_Tx,
 		(int)l_Adapter->m_TxErr,
+#if PACKET_TRUNCATION_CHECK
+		(int)l_Adapter->m_TxTrunc,
+#endif
 		(int)l_Adapter->m_Rx,
 		(int)l_Adapter->m_RxErr,
+#if PACKET_TRUNCATION_CHECK
+		(int)l_Adapter->m_RxTrunc,
+#endif
 		(int)l_Adapter->m_Extension.m_IrpQueue->size,
 		(int)l_Adapter->m_Extension.m_IrpQueue->max_size,
 		(int)IRP_QUEUE_SIZE,
 		(int)l_Adapter->m_Extension.m_PacketQueue->size,
 		(int)l_Adapter->m_Extension.m_PacketQueue->max_size,
-		(int)PACKET_QUEUE_SIZE
+		(int)PACKET_QUEUE_SIZE,
+		(int)l_Adapter->m_Extension.m_InjectQueue->size,
+		(int)l_Adapter->m_Extension.m_InjectQueue->max_size,
+		(int)INJECT_QUEUE_SIZE
 		);
 
 	      p_IRP->IoStatus.Information
@@ -1708,21 +1872,28 @@ TapDeviceHook (IN PDEVICE_OBJECT p_DeviceObject, IN PIRP p_IRP)
 	    }
 #endif
 
-	  case TAP_IOCTL_CONFIG_POINT_TO_POINT:
+	  case TAP_IOCTL_CONFIG_TUN:
 	    {
 	      if (l_IrpSp->Parameters.DeviceIoControl.InputBufferLength >=
-		  (sizeof (IPADDR) * 2))
+		  (sizeof (IPADDR) * 3))
 		{
 		  MACADDR dest;
 
-		  l_Adapter->m_PointToPoint = FALSE;
+		  l_Adapter->m_tun = FALSE;
 
 		  GenerateRelatedMAC (dest, l_Adapter->m_MAC, 1);
 
-		  l_Adapter->m_localIP =
-		    ((IPADDR*) (p_IRP->AssociatedIrp.SystemBuffer))[0];
-		  l_Adapter->m_remoteIP =
-		    ((IPADDR*) (p_IRP->AssociatedIrp.SystemBuffer))[1];
+		  l_Adapter->m_localIP =       ((IPADDR*) (p_IRP->AssociatedIrp.SystemBuffer))[0];
+		  l_Adapter->m_remoteNetwork = ((IPADDR*) (p_IRP->AssociatedIrp.SystemBuffer))[1];
+		  l_Adapter->m_remoteNetmask = ((IPADDR*) (p_IRP->AssociatedIrp.SystemBuffer))[2];
+
+		  // sanity check on network/netmask
+		  if ((l_Adapter->m_remoteNetwork & l_Adapter->m_remoteNetmask) != l_Adapter->m_remoteNetwork)
+		    {
+		      NOTE_ERROR ();
+		      p_IRP->IoStatus.Status = l_Status = STATUS_INVALID_PARAMETER;
+		      break;
+		    }
 
 		  COPY_MAC (l_Adapter->m_TapToUser.src, l_Adapter->m_MAC);
 		  COPY_MAC (l_Adapter->m_TapToUser.dest, dest);
@@ -1731,9 +1902,46 @@ TapDeviceHook (IN PDEVICE_OBJECT p_DeviceObject, IN PIRP p_IRP)
 
 		  l_Adapter->m_TapToUser.proto = l_Adapter->m_UserToTap.proto = htons (ETH_P_IP);
 
-		  l_Adapter->m_PointToPoint = TRUE;
+		  l_Adapter->m_tun = TRUE;
 
-		  CheckIfDhcpAndPointToPointMode (l_Adapter);
+		  CheckIfDhcpAndTunMode (l_Adapter);
+
+		  p_IRP->IoStatus.Information = 1; // Simple boolean value
+		}
+	      else
+		{
+		  NOTE_ERROR ();
+		  p_IRP->IoStatus.Status = l_Status = STATUS_INVALID_PARAMETER;
+		}
+	      
+	      break;
+	    }
+
+	  case TAP_IOCTL_CONFIG_POINT_TO_POINT: // Obsoleted by TAP_IOCTL_CONFIG_TUN
+	    {
+	      if (l_IrpSp->Parameters.DeviceIoControl.InputBufferLength >=
+		  (sizeof (IPADDR) * 2))
+		{
+		  MACADDR dest;
+
+		  l_Adapter->m_tun = FALSE;
+
+		  GenerateRelatedMAC (dest, l_Adapter->m_MAC, 1);
+
+		  l_Adapter->m_localIP =       ((IPADDR*) (p_IRP->AssociatedIrp.SystemBuffer))[0];
+		  l_Adapter->m_remoteNetwork = ((IPADDR*) (p_IRP->AssociatedIrp.SystemBuffer))[1];
+		  l_Adapter->m_remoteNetmask = ~0;
+
+		  COPY_MAC (l_Adapter->m_TapToUser.src, l_Adapter->m_MAC);
+		  COPY_MAC (l_Adapter->m_TapToUser.dest, dest);
+		  COPY_MAC (l_Adapter->m_UserToTap.src, dest);
+		  COPY_MAC (l_Adapter->m_UserToTap.dest, l_Adapter->m_MAC);
+
+		  l_Adapter->m_TapToUser.proto = l_Adapter->m_UserToTap.proto = htons (ETH_P_IP);
+
+		  l_Adapter->m_tun = TRUE;
+
+		  CheckIfDhcpAndTunMode (l_Adapter);
 
 		  p_IRP->IoStatus.Information = 1; // Simple boolean value
 		}
@@ -1791,7 +1999,7 @@ TapDeviceHook (IN PDEVICE_OBJECT p_DeviceObject, IN PIRP p_IRP)
 		  l_Adapter->m_dhcp_enabled = TRUE;
 		  l_Adapter->m_dhcp_server_arp = TRUE;
 
-		  CheckIfDhcpAndPointToPointMode (l_Adapter);
+		  CheckIfDhcpAndTunMode (l_Adapter);
 
 		  p_IRP->IoStatus.Information = 1; // Simple boolean value
 		}
@@ -1979,7 +2187,7 @@ TapDeviceHook (IN PDEVICE_OBJECT p_DeviceObject, IN PIRP p_IRP)
 	    p_IRP->IoStatus.Status = l_Status = STATUS_UNSUCCESSFUL;
 	    p_IRP->IoStatus.Information = 0;
 	  }
-	else if (!l_Adapter->m_PointToPoint && ((l_IrpSp->Parameters.Write.Length) >= ETHERNET_HEADER_SIZE))
+	else if (!l_Adapter->m_tun && ((l_IrpSp->Parameters.Write.Length) >= ETHERNET_HEADER_SIZE))
 	  {
 	    __try
 	      {
@@ -1989,6 +2197,18 @@ TapDeviceHook (IN PDEVICE_OBJECT p_DeviceObject, IN PIRP p_IRP)
 			     (unsigned char *) p_IRP->AssociatedIrp.SystemBuffer,
 			     l_IrpSp->Parameters.Write.Length);
 
+    //=====================================================
+    // If IPv4 packet, check whether or not packet
+    // was truncated.
+    //=====================================================
+#if PACKET_TRUNCATION_CHECK
+		IPv4PacketSizeVerify ((unsigned char *) p_IRP->AssociatedIrp.SystemBuffer,
+				      l_IrpSp->Parameters.Write.Length,
+				      FALSE,
+				      "RX",
+				      &l_Adapter->m_RxTrunc);
+#endif
+
 		NdisMEthIndicateReceive
 		  (l_Adapter->m_MiniportAdapterHandle,
 		   (NDIS_HANDLE) l_Adapter,
@@ -2011,7 +2231,7 @@ TapDeviceHook (IN PDEVICE_OBJECT p_DeviceObject, IN PIRP p_IRP)
 		p_IRP->IoStatus.Information = 0;
 	      }
 	  }
-	else if (l_Adapter->m_PointToPoint && ((l_IrpSp->Parameters.Write.Length) >= IP_HEADER_SIZE))
+	else if (l_Adapter->m_tun && ((l_IrpSp->Parameters.Write.Length) >= IP_HEADER_SIZE))
 	  {
 	    __try
 	      {
@@ -2022,6 +2242,18 @@ TapDeviceHook (IN PDEVICE_OBJECT p_DeviceObject, IN PIRP p_IRP)
 			      (unsigned char *) p_IRP->AssociatedIrp.SystemBuffer,
 			      l_IrpSp->Parameters.Write.Length);
 
+    //=====================================================
+    // If IPv4 packet, check whether or not packet
+    // was truncated.
+    //=====================================================
+#if PACKET_TRUNCATION_CHECK
+		IPv4PacketSizeVerify ((unsigned char *) p_IRP->AssociatedIrp.SystemBuffer,
+				      l_IrpSp->Parameters.Write.Length,
+				      TRUE,
+				      "RX",
+				      &l_Adapter->m_RxTrunc);
+#endif
+
 		NdisMEthIndicateReceive
 		  (l_Adapter->m_MiniportAdapterHandle,
 		   (NDIS_HANDLE) l_Adapter,
@@ -2197,7 +2429,7 @@ CompleteIRP (IN PIRP p_IRP,
   // component.
   //-------------------------------------------
 
-  if (p_PacketBuffer->m_SizeFlags & TP_POINT_TO_POINT)
+  if (p_PacketBuffer->m_SizeFlags & TP_TUN)
     {
       offset = ETHERNET_HEADER_SIZE;
       len = (int) (p_PacketBuffer->m_SizeFlags & TP_SIZE_MASK) - ETHERNET_HEADER_SIZE;
@@ -2303,15 +2535,16 @@ CancelIRP (TapExtensionPointer p_Extension,
     IoCompleteRequest (p_IRP, IO_NO_INCREMENT);
 }
 
-//====================================
-// Exhaust packet and IRP queues.
-//====================================
+//===========================================
+// Exhaust packet, IRP, and injection queues.
+//===========================================
 VOID
 FlushQueues (TapExtensionPointer p_Extension)
 {
   PIRP l_IRP;
   TapPacketPointer l_PacketBuffer;
-  int n_IRP=0, n_Packet=0;
+  InjectPacketPointer l_InjectBuffer;
+  int n_IRP=0, n_Packet=0, n_Inject=0;
 
   MYASSERT (p_Extension);
   MYASSERT (p_Extension->m_TapDevice);
@@ -2344,15 +2577,32 @@ FlushQueues (TapExtensionPointer p_Extension)
 	break;
     }
 
+  while (TRUE)
+    {
+      NdisAcquireSpinLock (&p_Extension->m_InjectLock);
+      l_InjectBuffer = QueuePop (p_Extension->m_InjectQueue);
+      NdisReleaseSpinLock (&p_Extension->m_InjectLock);
+      if (l_InjectBuffer)
+	{
+	  ++n_Inject;
+	  INJECT_PACKET_FREE(l_InjectBuffer);
+	}
+      else
+	break;
+    }
+
   DEBUGP ((
-	   "[%s] [TAP] FlushQueues n_IRP=[%d,%d,%d] n_Packet=[%d,%d,%d]\n",
+	   "[%s] [TAP] FlushQueues n_IRP=[%d,%d,%d] n_Packet=[%d,%d,%d] n_Inject=[%d,%d,%d]\n",
 	   p_Extension->m_TapName,
 	   n_IRP,
 	   p_Extension->m_IrpQueue->max_size,
 	   IRP_QUEUE_SIZE,
 	   n_Packet,
 	   p_Extension->m_PacketQueue->max_size,
-	   PACKET_QUEUE_SIZE
+	   PACKET_QUEUE_SIZE,
+	   n_Inject,
+	   p_Extension->m_InjectQueue->max_size,
+	   INJECT_QUEUE_SIZE
 	   ));
 }
 
@@ -2379,16 +2629,16 @@ SetMediaStatus (TapAdapterPointer p_Adapter, BOOLEAN state)
 
 
 //======================================================
-// If DHCP mode is used together with Point-to-point
-// mode, consider the fact that the P2P remote endpoint
-// might be equal to the DHCP masq server address.
+// If DHCP mode is used together with tun
+// mode, consider the fact that the P2P remote subnet
+// might enclose the DHCP masq server address.
 //======================================================
 VOID
-CheckIfDhcpAndPointToPointMode (TapAdapterPointer p_Adapter)
+CheckIfDhcpAndTunMode (TapAdapterPointer p_Adapter)
 {
-  if (p_Adapter->m_PointToPoint && p_Adapter->m_dhcp_enabled)
+  if (p_Adapter->m_tun && p_Adapter->m_dhcp_enabled)
     {
-      if (p_Adapter->m_dhcp_server_ip == p_Adapter->m_remoteIP)
+      if ((p_Adapter->m_dhcp_server_ip & p_Adapter->m_remoteNetmask) == p_Adapter->m_remoteNetwork)
 	{
 	  COPY_MAC (p_Adapter->m_dhcp_server_mac, p_Adapter->m_TapToUser.dest);
 	  p_Adapter->m_dhcp_server_arp = FALSE;
@@ -2404,7 +2654,8 @@ BOOLEAN
 ProcessARP (TapAdapterPointer p_Adapter,
 	    const PARP_PACKET src,
 	    const IPADDR adapter_ip,
-	    const IPADDR ip,
+	    const IPADDR ip_network,
+	    const IPADDR ip_netmask,
 	    const MACADDR mac)
 {
   //-----------------------------------------------
@@ -2420,7 +2671,8 @@ ProcessARP (TapAdapterPointer p_Adapter,
       && src->m_PROTO_AddressType == htons (ETH_P_IP)
       && src->m_PROTO_AddressSize == sizeof (IPADDR)
       && src->m_ARP_IP_Source == adapter_ip
-      && src->m_ARP_IP_Destination == ip)
+      && (src->m_ARP_IP_Destination & ip_netmask) == ip_network
+      && src->m_ARP_IP_Destination != adapter_ip)
     {
       ARP_PACKET *arp = (ARP_PACKET *) MemAlloc (sizeof (ARP_PACKET), TRUE);
       if (arp)
@@ -2442,14 +2694,14 @@ ProcessARP (TapAdapterPointer p_Adapter,
 	  COPY_MAC (arp->m_MAC_Destination, p_Adapter->m_MAC);
 	  COPY_MAC (arp->m_ARP_MAC_Source, mac);
 	  COPY_MAC (arp->m_ARP_MAC_Destination, p_Adapter->m_MAC);
-	  arp->m_ARP_IP_Source = ip;
+	  arp->m_ARP_IP_Source = src->m_ARP_IP_Destination;
 	  arp->m_ARP_IP_Destination = adapter_ip;
 
 	  DUMP_PACKET ("ProcessARP",
 		       (unsigned char *) arp,
 		       sizeof (ARP_PACKET));
 
-	  InjectPacket (p_Adapter, (UCHAR *) arp, sizeof (ARP_PACKET));
+	  InjectPacketDeferred (p_Adapter, (UCHAR *) arp, sizeof (ARP_PACKET));
 
 	  MemFree (arp, sizeof (ARP_PACKET));
 	}
@@ -2466,10 +2718,60 @@ ProcessARP (TapAdapterPointer p_Adapter,
 // seen as an incoming packet "arriving" on the interface.
 //===============================================================
 
+// Defer packet injection till IRQL < DISPATCH_LEVEL
 VOID
-InjectPacket (TapAdapterPointer p_Adapter,
-	      UCHAR *packet,
-	      const unsigned int len)
+InjectPacketDeferred (TapAdapterPointer p_Adapter,
+		      UCHAR *packet,
+		      const unsigned int len)
+{
+  InjectPacketPointer l_InjectBuffer;
+  PVOID result;
+
+  if (NdisAllocateMemoryWithTag (&l_InjectBuffer,
+				 INJECT_PACKET_SIZE (len),
+				 'IPAT') == NDIS_STATUS_SUCCESS)
+    {
+      l_InjectBuffer->m_Size = len;
+      NdisMoveMemory (l_InjectBuffer->m_Data, packet, len);
+      NdisAcquireSpinLock (&p_Adapter->m_Extension.m_InjectLock);
+      result = QueuePush (p_Adapter->m_Extension.m_InjectQueue, l_InjectBuffer);
+      NdisReleaseSpinLock (&p_Adapter->m_Extension.m_InjectLock);
+      if (result)
+	KeInsertQueueDpc (&p_Adapter->m_Extension.m_InjectDpc, p_Adapter, NULL);
+      else
+	INJECT_PACKET_FREE(l_InjectBuffer);
+    }
+}
+
+// Handle the injection of previously deferred packets
+VOID
+InjectPacketDpc(KDPC *Dpc,
+		PVOID DeferredContext,
+		PVOID SystemArgument1,
+		PVOID SystemArgument2)
+{
+  InjectPacketPointer l_InjectBuffer;
+  TapAdapterPointer l_Adapter = (TapAdapterPointer)SystemArgument1;
+  while (TRUE)
+    {
+      NdisAcquireSpinLock (&l_Adapter->m_Extension.m_InjectLock);
+      l_InjectBuffer = QueuePop (l_Adapter->m_Extension.m_InjectQueue);
+      NdisReleaseSpinLock (&l_Adapter->m_Extension.m_InjectLock);
+      if (l_InjectBuffer)
+	{
+	  InjectPacketNow(l_Adapter, l_InjectBuffer->m_Data, l_InjectBuffer->m_Size);
+	  INJECT_PACKET_FREE(l_InjectBuffer);
+	}
+      else
+	break;
+    }
+}
+
+// Do packet injection now
+VOID
+InjectPacketNow (TapAdapterPointer p_Adapter,
+		 UCHAR *packet,
+		 const unsigned int len)
 {
   MYASSERT (len >= ETHERNET_HEADER_SIZE);
 
@@ -2481,6 +2783,9 @@ InjectPacket (TapAdapterPointer p_Adapter,
       // TapDeviceHook/IRP_MJ_WRITE.
       //
       // The DDK docs imply that this is okay.
+      //
+      // Note that reentrant behavior could only occur if the
+      // non-deferred version of InjectPacket is used.
       //------------------------------------------------------------
       NdisMEthIndicateReceive
 	(p_Adapter->m_MiniportAdapterHandle,
@@ -2495,7 +2800,7 @@ InjectPacket (TapAdapterPointer p_Adapter,
     }
   __except (EXCEPTION_EXECUTE_HANDLER)
     {
-      DEBUGP (("[%s] NdisMEthIndicateReceive failed in InjectPacket\n",
+      DEBUGP (("[%s] NdisMEthIndicateReceive failed in InjectPacketNow\n",
 	       NAME (p_Adapter)));
       NOTE_ERROR ();
     }
@@ -2508,9 +2813,10 @@ InjectPacket (TapAdapterPointer p_Adapter,
 VOID ResetTapAdapterState (TapAdapterPointer p_Adapter)
 {
   // Point-To-Point
-  p_Adapter->m_PointToPoint = FALSE;
+  p_Adapter->m_tun = FALSE;
   p_Adapter->m_localIP = 0;
-  p_Adapter->m_remoteIP = 0;
+  p_Adapter->m_remoteNetwork = 0;
+  p_Adapter->m_remoteNetmask = 0;
   NdisZeroMemory (&p_Adapter->m_TapToUser, sizeof (p_Adapter->m_TapToUser));
   NdisZeroMemory (&p_Adapter->m_UserToTap, sizeof (p_Adapter->m_UserToTap));
 
@@ -2526,6 +2832,124 @@ VOID ResetTapAdapterState (TapAdapterPointer p_Adapter)
   p_Adapter->m_dhcp_bad_requests = 0;
   NdisZeroMemory (p_Adapter->m_dhcp_server_mac, sizeof (MACADDR));
 }
+
+#if ENABLE_NONADMIN
+
+//===================================================================
+// Set TAP device handle to be accessible without admin privileges.
+//===================================================================
+VOID AllowNonAdmin (TapExtensionPointer p_Extension)
+{
+  NTSTATUS stat;
+  SECURITY_DESCRIPTOR sd;
+  OBJECT_ATTRIBUTES oa;
+  IO_STATUS_BLOCK isb;
+  HANDLE hand = NULL;
+
+  NdisZeroMemory (&sd, sizeof (sd));
+  NdisZeroMemory (&oa, sizeof (oa));
+  NdisZeroMemory (&isb, sizeof (isb));
+
+  if (!p_Extension->m_CreatedUnicodeLinkName)
+    {
+      DEBUGP (("[TAP] AllowNonAdmin: UnicodeLinkName is uninitialized\n"));
+      NOTE_ERROR ();
+      return;
+    }
+
+  stat = RtlCreateSecurityDescriptor (&sd, SECURITY_DESCRIPTOR_REVISION);
+  if (stat != STATUS_SUCCESS)
+    {
+      DEBUGP (("[TAP] AllowNonAdmin: RtlCreateSecurityDescriptor failed\n"));
+      NOTE_ERROR ();
+      return;
+    }
+
+  InitializeObjectAttributes (
+    &oa,
+    &p_Extension->m_UnicodeLinkName,
+    OBJ_KERNEL_HANDLE,
+    NULL,
+    NULL
+    );
+
+  stat = ZwOpenFile (
+    &hand,
+    WRITE_DAC,
+    &oa,
+    &isb,
+    0,
+    0
+    );
+  if (stat != STATUS_SUCCESS)
+    {
+      DEBUGP (("[TAP] AllowNonAdmin: ZwOpenFile failed, status=0x%08x\n", (unsigned int)stat));
+      NOTE_ERROR ();
+      return;
+    }
+
+  stat = ZwSetSecurityObject (hand, DACL_SECURITY_INFORMATION, &sd);
+  if (stat != STATUS_SUCCESS)
+    {
+      DEBUGP (("[TAP] AllowNonAdmin: ZwSetSecurityObject failed\n"));
+      NOTE_ERROR ();
+      return;
+    }
+
+  stat = ZwClose (hand);
+  if (stat != STATUS_SUCCESS)
+    {
+      DEBUGP (("[TAP] AllowNonAdmin: ZwClose failed\n"));
+      NOTE_ERROR ();
+      return;
+    }
+
+  DEBUGP (("[TAP] AllowNonAdmin: SUCCEEDED\n"));
+}
+
+#endif
+
+#if PACKET_TRUNCATION_CHECK
+
+VOID
+IPv4PacketSizeVerify (const UCHAR *data, ULONG length, BOOLEAN tun, const char *prefix, LONG *counter)
+{
+  const IPHDR *ip;
+  int len = length;
+
+  if (tun)
+    {
+      ip = (IPHDR *) data;
+    }
+  else
+    {
+      if (length >= sizeof (ETH_HEADER))
+	{
+	  const ETH_HEADER *eth = (ETH_HEADER *) data;
+
+	  if (eth->proto != htons (ETH_P_IP))
+	    return;
+
+	  ip = (IPHDR *) (data + sizeof (ETH_HEADER));
+	  len -= sizeof (ETH_HEADER);
+	}
+      else
+	return;
+    }
+
+  if (len >= sizeof (IPHDR))
+    {
+      const int totlen = ntohs (ip->tot_len);
+
+      DEBUGP (("[TAP] IPv4PacketSizeVerify %s len=%d totlen=%d\n", prefix, len, totlen));
+
+      if (len != totlen)
+	++(*counter);
+    }
+}
+
+#endif
+
 //======================================================================
 //                                    End of Source
 //======================================================================
